


#import "MyController.h"

@implementation MyController

// Runs when we unpack from the nib
-(void)awakeFromNib
{
    songs=[[NSMutableArray alloc] init];    
    [songTable setDoubleAction:@selector(showWindow:)];
    [songTable registerForDraggedTypes:[NSArray arrayWithObjects:NSFilenamesPboardType,nil]];
    
    // When we quit the app, we want the song to stop playing, so we register to have stopPlaying:
    // be called when the proper notification comes in
   /* [[NSNotificationCenter defaultCenter] addObserver: songTable selector: @selector(stopPlaying:) name: NSApplicationWillTerminateNotification object:NSApp]*/
}

// Mandatory tableview data source method
- (int)numberOfRowsInTableView:(NSTableView *)tableView
{
    return [songs count];
}

// Mandatory tableview data source method
- (id)tableView:(NSTableView *)tableView objectValueForTableColumn:(NSTableColumn *)tableColumn row:(int)row
{
    if([[tableColumn identifier] isEqualToString: @"0"]) return [[songs objectAtIndex:row] getName];
    else return [[songs objectAtIndex:row] getState];
}

// Since the songs array is stored in this object, we need an accessor method to give us the particular
// song path for a given selected row in the table.  The tableview will call this method when trying
// to play a song.
- (Sieve *)songSelected
{    return [songs objectAtIndex:[songTable selectedRow]]; }

// Stop the table's rows from being editable when we double-click on them
- (BOOL)tableView:(NSTableView *)tableView shouldEditTableColumn:(NSTableColumn *)tableColumn row:(int)row
{    return NO; }

- (NSDragOperation)tableView:(NSTableView*)tv validateDrop:(id <NSDraggingInfo>)info proposedRow:(int)row proposedDropOperation:(NSTableViewDropOperation)op
{
    [tv setDropRow:[tv numberOfRows] dropOperation:NSTableViewDropAbove];
    return NSTableViewDropAbove;
}

- (BOOL)tableView:(NSTableView*)tv acceptDrop:(id <NSDraggingInfo>)info row:(int)row dropOperation:(NSTableViewDropOperation)op
{
    // Get the drag-n-drop pasteboard
    NSPasteboard *myPasteboard=[info draggingPasteboard];
    // What type of data are we going to allow to be dragged?  The pasteboard
    // might contain different formats
    NSArray *typeArray=[NSArray arrayWithObjects:NSFilenamesPboardType,nil];
    NSString *filePath,*availableType;
    NSArray *filesList;
    Sieve *s;
    int i;

    // find the best match of the types we'll accept and what's actually on the pasteboard
    availableType=[myPasteboard availableTypeFromArray:typeArray];
    // In the file format type that we're working with, get all data on the pasteboard
    filesList=[myPasteboard propertyListForType:availableType];
    // Insert the MP3 filenames into our songs array
    for (i=0;i<[filesList count];i++)
    {
        filePath=[filesList objectAtIndex:i];
        s = [[Sieve alloc] initWithWindowNibName:@"Sieve"];
        [s setPath:filePath];
        [songs insertObject:s atIndex:row+i];
        [s retain];
    }
    // We've updated our array.  Now reload the table.
    [songTable reloadData];
    // Select the last song.
    [songTable selectRow:row+i-1 byExtendingSelection:NO];
    return YES;
}

@end

