#import "Sieve.h"

@implementation Sieve

- (void)initialize:(id)string table:(id)table mixer:(Mixer *)m
{
    path = string;
    master = table;
    mix = m;
    name = [string lastPathComponent];
    state = @"Processing";
    [path retain];
    [name retain];
    [[self window] setTitle:name];
    vol = 1;
    NSString *filePath, *base = [NSHomeDirectory() stringByAppendingFormat:@"/Library/mosievius/"];
    base = [[base stringByAppendingFormat:name ] stringByAppendingFormat:@".mpf" ];
    presetArray = [[NSMutableArray alloc] initWithContentsOfFile:base];

    if(presetArray == nil) presetArray = [[NSMutableArray alloc] init];
    [presetTable setTarget:self];
    [presetTable setDoubleAction:@selector(setPreset:)];
    [presetTable reloadData];

    [[NSNotificationCenter defaultCenter] addObserver:self selector: @selector(cleanup:) name: NSApplicationWillTerminateNotification object:NSApp];
}
    
    
-(void)cleanup:(id)i
{
    NSString *filePath, *base = [NSHomeDirectory() stringByAppendingFormat:@"/Library/mosievius/"];
    base = [[base stringByAppendingFormat:name ] stringByAppendingFormat:@".mpf" ];
    [presetArray writeToFile:base atomically:YES];
}


- (id)getPath
{
    return path;
}

- (id)getName
{
    return name;
}


- (id)getState;
{
    return state;
}

- (IBAction)recieveMode:(id)sender
{
    BOOL tog = [sender intValue];
    int column = [sender selectedColumn];
    meat->toggle[column] = tog;
    if(tog) [[sender cellAtRow:0 column:column] setTitle:@"below"];
    else [[sender cellAtRow:0 column:column] setTitle:@"above"];
}

- (IBAction)recieveThresh:(id)sender
{
    meat->control([sender selectedColumn]) = [sender floatValue];
}

- (IBAction)power:(id)sender
{
    if([sender state] == 0) 
    {
        state = @"Ready";
        [sender setTitle:@"Start"];
    }
    else
    {
        state = @"Active";
        [sender setTitle:@"Stop"];
    }
    [master reloadData];
}

- (IBAction)memory:(id)sender
{
    meat->control(7) = [sender intValue];
}

- (IBAction)volume:(id)sender
{
    vol = [sender floatValue];
    return;
}

- (IBAction)presets:(id)sender
{
    int i;
    if([sender selectedColumn])
    {
        i = [presetArray count];
        if(!i) return;
        [presetArray removeObjectAtIndex:[presetTable selectedRow]];
        [presetTable reloadData];
        if(--i) [presetTable selectRow:i-1 byExtendingSelection:NO];
        return;
    }
    NSMutableArray *pre = [[NSMutableArray alloc] init];
    NSLog([presetName stringValue]);
    if([[presetName stringValue] isEqualToString:@""]) return;
    for(i = 0; i < 8; i++)
    {
        [pre insertObject:[NSNumber numberWithFloat:meat->control(i)] atIndex:i*2];
        [pre insertObject:[NSNumber numberWithInt:meat->toggle[i]] atIndex:i*2+1];
    }
    [pre insertObject:[presetName stringValue] atIndex:i*2];
    [presetArray insertObject:pre atIndex:[presetArray count]];
    [presetTable reloadData];
    [presetTable selectRow:[presetArray count]-1 byExtendingSelection:NO];
    [presetName setStringValue:@""];
}


-(void)setPreset:(id)sender
{
    int i;
    id p = [presetArray objectAtIndex:[presetTable selectedRow]];
    for(i = 0; i < 8; i++)
    {
        meat->control(i) = [[p objectAtIndex:i*2] floatValue];
        meat->toggle[i] = [[p objectAtIndex:i*2+1] floatValue];
    }
    for(i = 0; i < 7; i++)
    {
        [[setSliders cellAtRow:0 column:i] setFloatValue:meat->control(i)];
        [[setModes cellAtRow:0 column:i] setIntValue:meat->toggle[i]];
        if(meat->toggle[i]) [[setModes cellAtRow:0 column:i] setTitle:@"below"];
        else [[setModes cellAtRow:0 column:i] setTitle:@"above"];       
    }
    [setMemory setIntValue:meat->control(i)];
}


- (void)extraction:(id)table
{
    NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
    meat = new Sieve_back();
    meat->extract(path, WINSIZE);
    for(int i = 0; i < 8; i++)
    {
        NSSliderCell *r = [setSliders cellAtRow:0 column:i];
        [r setMinValue:meat->min(i)];
        [r setMaxValue:meat->max(i)];
        [r setFloatValue:meat->control(i)];
    }
    state = @"Ready";
    [table reloadData];
    [pool release];
    return;
}

- (void)tick:(float *)buffer OfSize:(int)size isKey:(BOOL)key
{
    int i;
    if(key)
    {
        int input;
        for(i = 0; i < 8; i++)
        {
            if(mix->control[i] != -1)
                meat->control(i) = meat->min(i)+(meat->max(i)-meat->min(i))*mix->control[i];
            if(mix->toggle[i] != -1)
                meat->control(i) = mix->toggle[i];
        }
    }
    meat->tick(buffer, size, vol);
    for(i = 0; i < 7; i++) 
    {
        NSSliderCell *r = [displayThresh cellAtRow:0 column:i];
        [r setFloatValue:meat->control(i)];
        [[setSliders cellAtRow:0 column:i] setFloatValue:meat->control(i)];
        [[setModes cellAtRow:0 column:i] setIntValue:meat->toggle[i]];
        if(meat->toggle[i]) [[setModes cellAtRow:0 column:i] setTitle:@"below"];
        else [[setModes cellAtRow:0 column:i] setTitle:@"above"];    
    }
}


- (int)numberOfRowsInTableView:(NSTableView *)tableView
{
    return [presetArray count];
}

- (id)tableView:(NSTableView *)tableView objectValueForTableColumn:(NSTableColumn *)tableColumn row:(int)row
{
    return [[presetArray objectAtIndex:row] objectAtIndex:16];
}

- (id)presetSelected
{
    return [presetArray objectAtIndex:[presetTable selectedRow]];
}

- (BOOL)tableView:(NSTableView *)tableView shouldEditTableColumn:(NSTableColumn *)tableColumn row:(int)row
{
	return NO;
}


@end
