/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/** 
    \class AMDF
    \brief Average Magnitude Difference Function

    Average Magnitude Difference Function
*/


#include "AMDF.h"

AMDF::AMDF()
{
  inSize_ = DEFAULT_WIN_SIZE;
  outSize_ = DEFAULT_WIN_SIZE;
}

AMDF::AMDF(unsigned int inSize)
{
  inSize_ = inSize;
  outSize_ = inSize;
}

AMDF::~AMDF()
{
}


void 
AMDF::process(fvec& in, fvec& out) 
{
  assert((in.size() == inSize_) && (out.size() == outSize_));  
  unsigned int i,j,k;
  float temp;
  
  for (i=0; i < inSize_; i++)
    {
      out(i) = 0.0;
      for (j=i, k=0; j < inSize_; j++, k++)
	{
	  temp = in(j) - in(k);
	  if (temp < 0) 
	    out(i) -= temp;
	  else 
	    out(i) += temp;
	}
    }
}


	



