#include <stdlib.h>			// standard libs
#include <stdio.h>			// standard I/O (for NULL)
#include <iostream.h>			// I/O streams
#include <math.h>			// math includes
#include <values.h>			// special values

class node
{
public:
    ~node() {}                    		// virtual distroyer
    void knnSearch() = 0;       		// tree search
    friend class knnTree;               
};
   
  
  
  
class branch : public node   
{
    int                 cut_dim;        // dim orthogonal to cutting plane
    ANNcoord            cut_val;        // location of cutting plane
    ANNcoord            cd_bnds[2];     // lower and upper bounds of
                                        // rectangle along cut_dim
    ANNkd_ptr           child[2];       // left and right children
public:
    ANNkd_split(                        // constructor
        int cd,                         // cutting dimension
        ANNcoord cv,                    // cutting value
        ANNcoord lv, ANNcoord hv,               // low and high values
        ANNkd_ptr lc=NULL, ANNkd_ptr hc=NULL)   // children
        {
            cut_dim     = cd;                   // cutting dimension
            cut_val     = cv;                   // cutting value
            cd_bnds[LO] = lv;                   // lower bound for rectangle
            cd_bnds[HI] = hv;                   // upper bound for rectangle
            child[LO]   = lc;                   // left child
            child[HI]   = hc;                   // right child
        }

    ~ANNkd_split()                      // destructor
        {
            if (child[LO]!= NULL && child[LO]!= KD_TRIVIAL) delete child[LO];
            if (child[HI]!= NULL && child[HI]!= KD_TRIVIAL) delete child[HI];
        }

    virtual void getStats(                      // get tree statistics
                int dim,                        // dimension of space
                ANNkdStats &st,                 // statistics
                ANNorthRect &bnd_box);          // bounding box
    virtual void print(int level, ostream &out);// print node
    virtual void dump(ostream &out);            // dump node

    virtual void ann_search(ANNdist);           // standard search routine
    virtual void ann_pri_search(ANNdist);       // priority search routine
};

  
  
  
   
   
class ANNkd_leaf: public ANNkd_node     // leaf node for kd-tree
{
    int                 n_pts;          // no. points in bucket
             bkt;            // bucket of points
public:
    ANNkd_leaf(                         // constructor
        int             n,              // number of points
        ANNidxArray     b)              // bucket
        {
            n_pts       = n;            // number of points in bucket
            bkt         = b;            // the bucket
        }

    ~ANNkd_leaf() { }                   // destructor (none)

    virtual void getStats(                      // get tree statistics
                int dim,                        // dimension of space
                ANNkdStats &st,                 // statistics
                ANNorthRect &bnd_box);          // bounding box
    virtual void print(int level, ostream &out);// print node
    virtual void dump(ostream &out);            // dump node

    virtual void ann_search(ANNdist);           // standard search routine
    virtual void ann_pri_search(ANNdist);       // priority search routine
};







   
class KNN_tree: public ANNpointSet {
protected:
    int				dim;		// dimension of space
    int				n_pts;		// number of points in tree
    int				bkt_size;	// bucket size
    ANNpointArray	pts;		// the points
    ANNidxArray		pidx;		// point indices (to pts)
    ANNkd_ptr		root;		// root of kd-tree

public:

    KNN_tree(
	fmatrix		pa,	
	int			n,			// number of points
	int			dd,			// dimension
	int			cn = 1);	// child nodes

    ~ANNkd_tree();			// tree destructor

    void knnSearch(			// approx k near neighbor search
	fvec	q,				// query point
	int		k,				// number of near neighbors to return
	int		*index,			// array to hold returned indexes
	);

};

