#include "BatchExtract.h"

using namespace std;
vector<string> featNames_;
vector<string> labelNames_;

struct ltstr
{
  bool operator()(const char* s1, const char* s2) const
  {
    return strcmp(s1, s2) < 0;
  }
};

inline
Collection 
readCollection(string name)
{
  string collectionstr = MARSYAS_MFDIR;
  collectionstr += "/collections/";
  collectionstr += name;
  cout << "Processing Collection " << collectionstr << endl;
  ifstream from(collectionstr.c_str());
  Collection l;
  from >> l;
  return l;
}

inline
string
makeName(string destName, const char *ext)
{
  string res;
  res += MARSYAS_MFDIR;
  res += "/featurefiles/";
  res += destName;
  res += ext;
  return res;
}

inline
streampos prepareStream(ofstream& os)
{
  streampos pos;
  pos = os.tellp();
  // create space for header
  unsigned int i;
  for (i=0; i < 6; i ++)
    os << "                           " << endl;
  return pos;
}

inline
void writeHeaderFeatures(ofstream& os, streampos pos, string destName, int rows, int cols)
{
  os.seekp(pos);
  os << "# MARSYAS Features" << endl;
  os << "# name: " << destName << endl;
  os << "# type: matrix" << endl;
  os << "# rows: " << rows << endl;
  os << "# columns: " << cols << endl;
}

inline
void writeHeaderLabels(ofstream& os, streampos pos, string destName, int rows, int cols)
{
  os.seekp(pos);
  os << "# MARSYAS Labels" << endl;
  os << "# name: " << destName << endl;
  os << "# type: matrix" << endl;
  os << "# rows: " << rows << endl;
  os << "# columns: " << cols << endl;
}

inline
int myextract(string collection, string destName, string extractor, int winSize,
            int hopSize)
{
  string mffName = makeName(destName, ".mff");
  string mlfName = makeName(destName, ".mlf");
  string mdfName = makeName(destName, ".mdf");

  ofstream fos(mffName.c_str());
  streampos fpos = prepareStream(fos);
  ofstream los(mlfName.c_str());
  streampos lpos = prepareStream(los);
    
  int trows = 0;			// total number of rows
  int rows = 0;
  int cols = 0;
  int lcols = 2;			// label cols
  int count =0;
  unsigned int i;

  Collection l = readCollection(collection);
  cout << "read " << collection << endl;      
  string l0name = collection;
  l0name = l0name.substr(0, l0name.size()-3); // remove .mf
  labelNames_.push_back(l0name);
  MSignal *src;

  int lsize = l.size();
  System *sys_;
  Extractor* extr;
  for (i=0; i<lsize; i++)
  {
    string fromName = l.entry(i);
    cerr << "Extract from " << fromName << endl;
    Util util;
    src = util.sfopen(l.entry(i), MRS_SF_READ);
    src->initWindow(winSize, hopSize, 0, 0);
		
    if(extractor == "SFFT") sys_ = new Spectral1(src->winSize(), src->zeroSize()); 
    else if(extractor == "PITCH") sys_ = new Pitch(src->winSize());
    else if(extractor == "PITCHES") sys_ = new Pitches(src->winSize(), 1); 
    else if(extractor == "LPC") sys_ = new LPC(src, 10);
    else if(extractor == "MFCC") sys_ = new MFCC(src->winSize(), src->zeroSize());
		else if(extractor == "DFCC") sys_ = new DFCC(src->winSize(), 40, 20);
    else
		{
			cerr << "BatchExtract: extractor " << extractor << " not supported" << endl;
			exit(1);
    }
    extr = new FeatExtractor(src, sys_); 
    FeatMatrix* featMatrix = new FeatMatrix(src->iterations(), sys_->outSize());
    extr->extract(*featMatrix);
    featMatrix->setName(fromName);
    featMatrix->printHeader(false);

    fos << (fmatrix)(*featMatrix);
    int nrows = featMatrix->rows();
    imatrix labels(nrows, 2);
    unsigned int r;
    labels.printHeader(false);
    for (r=0; r < nrows; r++)
    {
      labels(r,0) = 0;
      labels(r,1) = count;
    }        
          
    los << labels; 
  
    featNames_ = featMatrix->featNames();
  
    rows = featMatrix->rows();
    cols = featMatrix->cols();

    trows += rows;
    count ++;

    delete src;
    delete extr;
    //delete featMatrix;
  }

  l = readCollection(collection);
  cout << "read " << collection << endl;      
  for (i=0; i<l.size(); i++)
  {
    string l1name = l.entry(i);
	labelNames_.push_back(l1name);
  }
  
  cout << "Wrote " << mffName << endl;

  // Write description file
  ofstream dos(mdfName.c_str());
  dos << "# MARSYAS Description\n" << endl;
  
  unsigned int featSize;
  featSize = featNames_.size();
  unsigned int labelSize;
  labelSize = labelNames_.size();

  dos << "# Feature names " << endl;
  dos << featSize << endl << endl;
  for (i=0; i < featSize; i++)
    dos << featNames_[i] << endl;
  dos << endl;
  dos << "# Label names " << endl;
  dos << labelSize << endl;
  for (i=0; i < labelSize; i++)
    dos << labelNames_[i] << endl;
  
  // write .mff header 
  writeHeaderFeatures(fos, fpos, destName, trows, cols);
  
  // write .mlf header 
  writeHeaderLabels(los, lpos, destName, trows, lcols);
  
  return 0;
}


BatchExtract::BatchExtract(
  string collection,
  string winSizeStr,
  string hopSizeStr)
{

  collectName = collection;
  winSizeStr_ = winSizeStr;
  hopSizeStr_ = hopSizeStr;
  winSize = atoi(winSizeStr.c_str());
  hopSize = atoi(hopSizeStr.c_str());

}



FeatMatrix *
BatchExtract::extract(
  string extractor,
  int **startnum)
{
  string dstring, lstring, fstring, collectStr, tmpstring, extCollectStr;
  extCollectStr = collectName;
  extCollectStr += winSizeStr_;
  extCollectStr += "-";
  extCollectStr += hopSizeStr_;
  extCollectStr += extractor;
  fstring = makeName(extCollectStr, ".mff");
  lstring = makeName(extCollectStr, ".mdf");
  dstring = makeName(extCollectStr, ".mlf");
  struct stat stbuf;

  if(stat(fstring.c_str(), &stbuf) ||
     stat(dstring.c_str(), &stbuf) ||
     stat(lstring.c_str(), &stbuf))
  {
    cout << "Feature files for '" << collectName <<
        "' do not exist" << endl;
    cout << "Creating feature files ...." << endl;
    
    tmpstring=MARSYAS_MFDIR;
    tmpstring += "/collections/";
    tmpstring += collectName;
    if(stat(tmpstring.c_str(), &stbuf))
    {
      cout << "shuffle: collection '"<<tmpstring<<".mf' does not exist"<<endl;
      exit(1);
    } 
    myextract(collectName, extCollectStr, extractor, winSize, hopSize);
  } 
  
  cout << "Reading feauturefiles for " << collectName << endl;
  tmpstring=makeName(extCollectStr, "");
  collectRes = new FeatMatrix();
  collectRes->read(tmpstring);
  
  if(*startnum == NULL)
  {  
    unsigned int tmp, i, j, collectionSize;
    string str;
    ifstream dis(lstring.c_str());
    for (i=0; i < 6; i++)
      dis >> str;
    dis >> tmp;
    for (i=0; i < tmp + 3; i++)
      dis >> str;
    dis >> tmp;
    collectionSize = tmp-1;
  
    *startnum = new int[collectionSize];
    j=0;
    for(i=0; i<collectionSize; i++)
    {
      (*startnum)[i]=j;
      if(i != collectionSize - 1) while(collectRes->labelNum(j, 1) == i) j++;
    }
  }
 return collectRes;
}



FeatMatrix *
BatchExtract::extract(
  string extractor)
{
  string dstring, lstring, fstring, collectStr, tmpstring, extCollectStr;
  extCollectStr = collectName;
  extCollectStr += winSizeStr_;
  extCollectStr += "-";
  extCollectStr += hopSizeStr_;
  extCollectStr += extractor;
  fstring = makeName(extCollectStr, ".mff");
  lstring = makeName(extCollectStr, ".mdf");
  dstring = makeName(extCollectStr, ".mlf");
  struct stat stbuf;

  if(stat(fstring.c_str(), &stbuf) ||
     stat(dstring.c_str(), &stbuf) ||
     stat(lstring.c_str(), &stbuf))
  {
    cout << "Feature files for '" << collectName <<
        "' do not exist" << endl;
    cout << "Creating feature files ...." << endl;
    
    tmpstring=MARSYAS_MFDIR;
    tmpstring += "/collections/";
    tmpstring += collectName;
    if(stat(tmpstring.c_str(), &stbuf))
    {
      cout << "shuffle: collection '"<<tmpstring<<".mf' does not exist"<<endl;
      exit(1);
    } 
    myextract(collectName, extCollectStr, extractor, winSize, hopSize);
  } 
  
  cout << "Reading feauturefiles for " << collectName << endl;
  tmpstring=makeName(extCollectStr, "");
  collectRes = new FeatMatrix();
  collectRes->read(tmpstring);
  
 return collectRes;
}









BatchExtract::~BatchExtract()
{delete collectRes;}



