/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/


/** 
    \class BeatHistoFeatures
    \brief Calculate features of beat histogram

    Calculate features for beat histogram.
*/





#include "BeatHistoFeatures.h"

BeatHistoFeatures::BeatHistoFeatures()
{
  inSize_ = DEFAULT_WIN_SIZE;
  outSize_ = 6;
  numPeaks_ = 3;

}

BeatHistoFeatures::~BeatHistoFeatures()
{
}


BeatHistoFeatures::BeatHistoFeatures(unsigned int inSize)
{
  inSize_ = inSize;
  outSize_ = 6;
  numPeaks_ = 3;
}


void 
BeatHistoFeatures::process(fvec& in, fvec& out) 
{
  assert((in.size() == inSize_) && (out.size() == outSize_));  
  unsigned int i,k,j; 
  
  float max;
  max = 0.0; 
  fvec maximums;
  ivec maxPositions;
  maximums.create(numPeaks_);
  maxPositions.create(numPeaks_);
  
  
  float sum = 0.0;
  for (i=1; i < in.size()-1; i++) 
    {
      sum += in(i);
    }

  int bw = 5;

  
  for (k=0; k < numPeaks_; k++)
    {
      max = 0.0;
      for (i=1; i < in.size()-1; i++) 
	{
	  if ((in(i) > max) && (in(i) > in(i-1)) && (in(i) > in(i+1)))
	    {
	      max = in(i);
	      maximums(k) = max;
	      maxPositions(k) = i;
	    }
	}
      in(maxPositions(k)) = 0.0;
      if (maxPositions(k) > bw)
	for (j= maxPositions(k) - bw; j < maxPositions(k) + bw; j++)
	  in(j) = 0.0;
    }
  
  
  out(0) = maximums(0) / sum;
  out(1) = maximums(1) / sum;
  out(2) = (float)maximums(1) / (float)maximums(0);
  out(3) = (float)maxPositions(0);
  out(4) = (float)maxPositions(1);
  out(5) = sum;
  
}


	
