/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/** 
    \class ClassifierDictionary.h
    \brief Dictionary of supported extractors

    ClassifierDictionary returns classifiers
based on a string argument which is the classifier name. 
*/



#include "ClassifierDictionary.h"


ClassifierDictionary::ClassifierDictionary()
{
  gs_ = new Gaussian();
  gmm1_ = new GMM(1);
  gmm2_ = new GMM(2);
  gmm3_ = new GMM(3);
  gmm4_ = new GMM(4);
  gmm5_ = new GMM(5);
  knn1_ = new KNN(1);
  knn3_ = new KNN(3);
  knn5_ = new KNN(5);

  /* Create classifier dictionary */ 
  classifiers_["GS"] = gs_;
  classifiers_["GMM1"] = gmm1_;
  classifiers_["GMM2"] = gmm2_;
  classifiers_["GMM3"] = gmm3_;
  classifiers_["GMM4"] = gmm4_;
  classifiers_["GMM5"] = gmm5_;
  classifiers_["KNN1"] = knn1_;
  classifiers_["KNN3"] = knn3_;
  classifiers_["KNN5"] = knn5_;

  prepareNames();
}


void 
ClassifierDictionary::prepareNames()
{
  classifierNames_.reserve(100);
  classifierNames_.push_back("GS");
  classifierNames_.push_back("GMM1");
  classifierNames_.push_back("GMM2");
  classifierNames_.push_back("GMM3");
  classifierNames_.push_back("GMM4");
  classifierNames_.push_back("GMM5");
  classifierNames_.push_back("KNN1");
  classifierNames_.push_back("KNN3");
  classifierNames_.push_back("KNN5");
}
			   

ClassifierDictionary::~ClassifierDictionary()
{
  delete gs_;
  delete gmm1_;
  delete gmm2_;
  delete gmm3_;
  delete gmm4_;
  delete gmm5_;
  delete knn1_;
  delete knn3_;
  delete knn5_;
}


Classifier* 
ClassifierDictionary::getClassifier(string classifierstr)
{
  map<const char*, Classifier *, ltstr>::iterator cur;
  const char *ch = classifierstr.c_str();
  cur = classifiers_.find(ch);
  Classifier* it;
  if (cur == classifiers_.end())
    {
      cerr << "Classifier " << classifierstr << " is not supported\n" << endl;
      return NULL;
    }
  else
    {
      it = classifiers_[classifierstr.c_str()];
    }
  return it;
}


ostream&
operator<<(ostream& o, const ClassifierDictionary& it)
{
  unsigned int i;
  for (i=0; i < it.classifierNames_.size(); i++)
    {
      o << it.classifierNames_[i] << endl;
    }
  return o;
}



