/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/** 
    \class Clusterer.h
    \brief Base class for clusterers

    Base class that all clusterers inherit. Unlike Classifiers 
that use supervised learning Clusteres use unsupervised learning. 
Therefore the feature vectors are unlabeled and the only optional 
input is the number of desired clusters. 
*/

#include "Clusterer.h"

Clusterer::Clusterer()
{
  numClusters_ = 0;
}


Clusterer::~Clusterer()
{
}


void 
Clusterer::init(unsigned int numClusters)
{
  numClusters_ = numClusters;
}


void 
Clusterer::clusterCollection(string collectionstr, string namesstr)
{
  unsigned int i;
  unsigned int cl;


  ifstream from(collectionstr.c_str());
  Collection l;
  from >> l;
  
  Collection nl;
  nl.read(namesstr);

  vector <FeatMatrix *> matrices;
  matrices.reserve(200);
  for (i=0; i < l.size(); i++)
    {
      matrices.push_back(new FeatMatrix());
      matrices[i]->read(l.entry(i));
      cout << "Reading FeatMatrix file :" << l.entry(i) << endl;
    }
  FeatMatrix trainMatrix(matrices);
  cluster(&trainMatrix);
  if (trainMatrix.rows() != l.size())
    {
      cerr << "Trouble Collection size != trainMatrix\n" << endl;
    }
  unsigned int size;

  fvec window(1024);
  
  for (cl =0; cl < trainMatrix.labelSize(0); cl++)
    {
      cout << "****  CLASS " << cl << " ***** " << endl;
      size = 0;
      for (i=0; i < l.size(); i++)
	{
	  if (trainMatrix.labelNum(i,0) == cl)
	    {
	      //cout << l.entry(i) << endl;
	      cout << nl.entry(i) << endl;
	      Util util;
	      MSignal *src;
	      src = util.sfopen(nl.entry(i), MRS_SF_READ);
	      src->initWindow(1024,1024,0,0);
	      Audio audio(src->srate(),1, MRS_AUDIO_PLAY, src->winSize());
	      for (unsigned int i=0; i< src->iterations(); i++)
		{
		  src->get(window,i);
		  audio.put(window,i);
		}	      
	      
	      size++;
	    }
	}
      if (size == 0) 
	cout << "WARNING: Class " << cl << " is empty" << endl;
    }
  
  trainMatrix.write("clustered.mf");
}
