/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/


/**
   \class  CollectionShuffler
   \brief  Interface for segmentors

   Abstract base class used as interface for segmentors. CollectionShufflers 
   as input a FeatMatrix and produce a temporal segmentation 
   of the FeatMatrix into regions in time stored as 1.0 and 0.0 
   in a fvec. 
*/


#include "CollectionShuffler.h"

CollectionShuffler::CollectionShuffler()
{
}


CollectionShuffler::~CollectionShuffler()
{
}

void
CollectionShuffler::init(FeatMatrix* cmatrix)
{
  cmatrix_ = cmatrix;
}

void 
CollectionShuffler::shuffle(FeatMatrix& matrix, ivec &mapping, int winNum)
{
  float dist;
  float min;
  unsigned int minIndex =0;
  int r,s,c,t;
  unsigned int cols;
  cols = cmatrix_->cols();
  unsigned int templSize;
  templSize = matrix.rows();
  unsigned int collectSize;
  collectSize = cmatrix_->rows();
    
  ivec marked(collectSize);
  fvec vars(cols);
  fvec covars(cols);
  vars = matrix.varRow();
  fvec v1(cols);
  fvec v2(cols);
  fvec *V1;
  fvec *V2;
  V1=new fvec[winNum](cols);
  V2=new fvec[winNum](cols);
  int z;

  for (c=0; c < (int)cols; c++)
    {
      if (vars(c) <= 0.000001) 
	covars(c) = 1.0;		// No variance
      else
	covars(c) = 1.0 / vars(c);
    }
  covars.setval(1.0);
  marked.setval(0);
  cout << "mapping..." << endl;
  int percent=0;
  cout << "0%" << endl;

  int distances[winNum];

  if (winNum==1) for (r=0; r < (int)templSize; r++)
    {
      if(percent < (r*1000/templSize))
      {
	    percent=r*1000/templSize;
        cout << (float)percent/10.0 << "%" << endl;
      }
      min = 100000000;
      v1 = matrix.row(r);
      for (s=0; s < (int)collectSize; s++)
	{
	  v2 = cmatrix_->row(s);
	  dist = mahalanobis(v1, v2, covars);
	  if ((dist < min)&&(marked(s) != 1))
	    {
		  min = dist;
		  minIndex = s;
	    }
	}
      mapping(r) = minIndex;
      cout << min << endl;
    }
  else  for (r=0; r < (int)templSize; r++)
    {
      if(percent< (r*1000/templSize))
      {
	percent=r*1000/templSize;
        cout << (float)percent/10.0 << "%" << endl;
      }
      min = 100000000;
      for(z=0; z<winNum && (r+z)<templSize; z++) V1[z] = matrix.row(r+z);
      for (s=0; s < (int)collectSize; s++)
	{
	  for(z=0; z<winNum && (s+z)<collectSize; z++) V2[z] = cmatrix_->row(s+z);
	  dist=0;
	  for(z=0; z<winNum && (s+z)<collectSize && (r+z)<templSize; z++) dist += mahalanobis(V1[z], V2[z], covars);
	  if ((dist < min) && z==winNum)
	    {
		  min = dist;
		  minIndex = s;
	    }
	}
      for(z=0; z<winNum && (r+z)<templSize; z++) mapping(r+z) = minIndex+z;
    }





}
