//Discrete Cepstrum Coefficients
//with regularization and cloud method


#include "DFCC.h"

DFCC::DFCC()
{
  inSize_ = 20;
	maxorder_ = 10;
  outSize_ = 42;
	init();
}

DFCC::DFCC(unsigned int maxpartials, unsigned int maxorder)
{
	maxpartials_ = maxpartials;
  maxorder_ = maxorder;
  inSize_ = maxpartials * 2;
  outSize_ = maxorder*4+2;
	automatic_ = false;
	init();
}

DFCC::DFCC(unsigned int inSize, unsigned int maxpartials, unsigned int maxorder)
{
	maxpartials_ = inSize_/2;
  maxorder_ = maxorder;
  inSize_ = inSize;
  outSize_ = maxorder*4+2;
	automatic_ = true;
	pitch_ = new Pitch(inSize_);
	init();
}

DFCC::~DFCC()
{
delete pitch_;
}

void
DFCC::init()
{
  featSize_ = maxorder_;

	spectrum_.create(inSize_/8);
  pit.create(2);

  A.create(maxorder_, maxorder_);
	M.create(inSize_/2, maxorder_);
  L.create(maxorder_, maxorder_);
	Li.create(maxorder_, maxorder_);
	noise_.create(maxorder_);
	
	b.create(maxorder_);
	d.create(maxorder_);
	y.create(maxorder_);
	f.create(inSize_/8);
	a.create(inSize_/8);
	h.create(inSize_/8);
	sub.create(inSize_/4);
	
	fft_ = new MagFFT(inSize_/4);
	ham_ = new Hamming(inSize_/4, 0);
	fvec temp(inSize_/4);
	temp.setval(1);
	ham_->process(temp, temp);
	int i;
	for(i = 0, hamsum_ = 0, hamsum2_ = 0; i < inSize_/4; i++)
	{
	  hamsum_ += temp(i);
	  hamsum2_ += temp(i) * temp(i);
	}
}

void
DFCC::set(unsigned int inSize, unsigned int order)
{
  inSize_ = inSize;
  outSize_ = order+2;
}

float
DFCC::get(fvec& envelope, float f)
{
  int i;
  float out = envelope(0);
	for(i = 1; i < maxorder_; i++) out += 2*envelope(i)*cos(f*i);
	return out;
}


inline int round(float r)
{
  if(r - floor(r) <= 0.5 ) return (int)floor(r);
  return (int)ceil(r);
}


fvec&
DFCC::noise()
{
  return noise_;
}

float
DFCC::pitch()
{
  return pit(0);
}


void 
DFCC::process(fvec& in, fvec& out)
{
  int npartials, rpartials, i, j, k, l, noodle;
	float diff, lowdiff, pt;
	
	pitch_->process(in, pit);
	if(pit(0) == 0) return;
	
	pt = pit(0)/4;
	out(0) = pit(0);
	out(1) = pit(1);
	for(noodle = 0; noodle < 4; noodle++)
	{
	for(i=0; i<inSize_/4; i++) sub(i) = in(i+inSize_*noodle/4);
	ham_->process(sub, sub);
	fft_->process(sub, spectrum_);

  // get noise envelope
	for(k = 0, l = 0; k < round(pt)-1; k++) if(spectrum_(k)) f(l++) = k;
	for(i = 1; i < floor(inSize_/(8*pt)); i++)
	  for(k = round(i*pt)+2; k < round((i+1)*pt)-1; k++) if(spectrum_(k)) f(l++) = k;
	for(k = round(i*pt)+2; k < inSize_/8; k++) if(spectrum_(k)) f(l++) = k;
	for(i = 0; i < l; i++)
	{
	  a(i) = log(spectrum_(f(i))*spectrum_(f(i))/hamsum2_) + .577215664901532;
		f(i) *= 8 * PI / inSize_;
	}
	h.setval(1);
	noise_.setval(0);
  if(l > 1) wls(l, 16, f, a, h, noise_, l*.0005);

	/*for(i = 1, l = 0; i < inSize_/8; i++)
	if(spectrum_(i)) {
	  a(l) = log(spectrum_(i)*spectrum_(i)/hamsum2_) + .577215664901532;
		f(l) = 8 * i * PI / inSize_;	 
		l++; 
  }
  out.setval(0);
  wls(l, 10, f, a, h, out, l*.005);
	*/
  // choose peaks
	f.setval(0);
	a.setval(0);
	rpartials = (int)floor(inSize_/(8*pt));
	if(rpartials > 20) rpartials = 20;
	for(i = 1, l = 0, l ; i <= rpartials; i++, l++)
	{
		for(f(l) = (int)(pt*i), k = f(l)-3; k < (int)(pt*i)+3; k++)
			if(spectrum_(k) > spectrum_(k-1) && spectrum_(k) > spectrum_(k+1) && spectrum_(f(l)) <= spectrum_(k)) f(l) = k;
		k = f(l);
		if(spectrum_(k) > spectrum_(k-1) && spectrum_(k) > spectrum_(k+1))
		{				    
			diff = 0.5 * (spectrum_(k-1)-spectrum_(k+1))/(spectrum_(k-1)-2*spectrum_(k)+spectrum_(k+1));
			f(l) = k + diff;
			a(l) = log(spectrum_(k) - .25 * (spectrum_(k-1) - spectrum_(k+1)) * diff);
		}
		else a(l) = log(spectrum_(f(l)));
    f(l) *= 8 * PI / inSize_;
		if(a(l) <= hamsum2_ * get(noise_, f(l)) / (2 * hamsum_)) l--;
	}
	rpartials = l;


	fvec temp(maxorder_), temp1(maxorder_);
	if(l > 1) {
	wls(rpartials, maxorder_, f, a, h, temp1, 0.6);
  for(i = 0; i < rpartials; i++) h(i) = get(temp1, f(i))*hamsum_*hamsum_/(4*hamsum2_*get(noise_, f(i)));
	wls(rpartials, maxorder_, f, a, h, temp, 0.6);
	for(i = 0; i < maxorder_; i++) out(i+2+maxorder_*noodle) = temp(i); }
	/*
  ofstream as("a",std::ios_base::app), fs("fs",std::ios_base::app), spec("spec",std::ios_base::app), oenv("oenv",std::ios_base::app),env("env",std::ios_base::app), noise("noise",std::ios_base::app);
  for(i = 0; i < 20; i++)
  {
    as << a(i) << endl;
	  fs << f(i)*inSize_/(8*PI) << endl;
  }
	for(i = 1; i < 256; i++)
	{
	  spec << log(spectrum_(i)) << endl;
	  env << get(temp, i*8*PI/inSize_) << endl;
	  oenv << get(temp1, i*8*PI/inSize_) << endl;
		noise << hamsum2_*get(noise_, i*8*PI/inSize_)/(2*hamsum_) << endl;
	}*/
	}
}


//solve equation Ac = b where A is factored to LDL* = b
void
DFCC::wls(int partials, int order, fvec& freq, fvec& amp, fvec& weight, fvec& o, float g)
{
  int i, j, k, sum;

  M.setval(0);
	for(i = 0; i < partials; i++) M(i, 0) = 1;
	for(i = 0; i < partials; i++)
	  for(j = 1; j < order; j++) M(i, j) = 2*cos(freq(i)*j);	

  fmatrix temp(order, order), temp1(order, order);
	for(i = 0; i < order; i++)
	  for(j = 0; j <= i; j++)
		  for(k = 0; k < partials; k++) temp(i, j) = temp(j, i) += weight(k)*M(k, i)*M(k, j); 
	for(i = 0; i < order; i++) temp(i, i) += 2*i*i*g;
	
	b.setval(0);
	for(i = 0; i < order; i++)
	  for(j = 0; j < partials; j++) b(i) += weight(j) * amp(j) * M(j, i);

	temp.invert(temp1);
	o.setval(0);
	for(i = 0; i < order; i++)
	 for(j=0; j<order; j++) o(i)+= temp(i, j)*b(j);
}










