//Discrete Cepstrum Coefficients
//with regularization and cloud method


#include "DFCC.h"

DFCC::DFCC()
{
  inSize_ = 20;
  outSize_ = 42;
	init();
}

DFCC::DFCC(unsigned int maxpartials, unsigned int maxorder)
{
	maxpartials_ = maxpartials;
  maxorder_ = maxorder;
  inSize_ = maxpartials * 2;
  outSize_ = maxorder+2;
	automatic_ = false;
	init();
}

DFCC::DFCC(unsigned int inSize, unsigned int maxpartials, unsigned int maxorder)
{
	maxpartials_ = inSize_/2;
  maxorder_ = maxorder;
  inSize_ = inSize;
  outSize_ = maxorder+2;
	automatic_ = true;
	pitch_ = new Pitch(inSize_);
	init();
}

DFCC::~DFCC()
{
delete pitch_;
}

void
DFCC::init()
{
  featSize_ = maxorder_;

  peaks_.create(inSize_/2);
	amps_.create(inSize_/2);
	spectrum_.create(inSize_/2);
  pit.create(2);

  A.create(maxorder_, maxorder_);
	M.create(inSize_/2, maxorder_);
  L.create(maxorder_, maxorder_);
	Li.create(maxorder_, maxorder_);
	noise_.create(maxorder_);
	
	b.create(maxorder_);
	d.create(maxorder_);
	y.create(maxorder_);
	f.create(inSize_/2);
	a.create(inSize_/2);
	h.create(inSize_/2);
	
	Hamming ham(inSize_, 0);
	fvec temp(inSize_);
	temp.setval(1);
	ham.process(temp, temp);
	int i;
	for(i = 0, hamsum_ = 0, hamsum2_ = 0; i < inSize_; i++)
	{
	  hamsum_ += temp(i);
	  hamsum2_ += temp(i) * temp(i);
	}
}

void
DFCC::set(unsigned int inSize, unsigned int order)
{
  inSize_ = inSize;
  outSize_ = order+2;
}

float
DFCC::get(fvec& envelope, float f)
{
  unsigned int i;
  float out = envelope(0);
	for(i = 1; i < maxorder_; i++) out += 2*envelope(i)*cos(f*i);
	return out;
}


inline int round(float r)
{
  if(r - floor(r) <= 0.5 ) return (int)floor(r);
  return (int)ceil(r);
}


fvec&
DFCC::noise()
{
  return noise_;
}

float
DFCC::pitch()
{
  return pit(0);
}


void 
DFCC::process(fvec& in, fvec& out)
{
  int npartials, rpartials, i, j, k, l;
	float diff, lowdiff;
	
	pitch_->process(in, pit);
	peaks_ = pitch_->getPeaks();
	amps_ = pitch_->getAmps();
	spectrum_ = pitch_->getSpectrum();

  if(pit(0) == 0) return;

  // get noise envelope
	for(k = 0, l = 0; k < round(pit(0))-1; k+=2) if(spectrum_(k)) f(l++) = k;
	for(i = 1; i < floor(inSize_/(2*pit(0))); i++)
	  for(k = round(i*pit(0))+2; k < round((i+1)*pit(0))-1; k+=2) if(spectrum_(k)) f(l++) = k;
	for(k = round(i*pit(0))+2; k < inSize_/2; k+=2) if(spectrum_(k)) f(l++) = k;
	for(i = 0; i < l; i++)
	{
	  a(i) = log(spectrum_(f(i))*spectrum_(f(i))/hamsum2_) + .577215664901532;
		f(i) *= 2 * PI / inSize_;
	}
	h.setval(1);
  wls(l, 16, f, a, h, noise_, l*.005);
/*	for(i = 1, l = 0; i < inSize_/2; i++)
	if(spectrum_(i)) {
	  a(l) = log(spectrum_(i)*spectrum_(i)/hamsum2_) + .577215664901532;
		f(l) = 2 * i * PI / inSize_;	 
		l++; 
  }
out.setval(0);
  wls(l, 16, f, a, h, out, l*.0005);
*/
  // choose peaks
	f.setval(0);
	a.setval(0);
	npartials = pitch_->numberOfPartials();
	rpartials = (int)floor(inSize_/(2*pit(0)));
	for(i = 1, l = 0, l ; i <= rpartials; i++, l++)
	{
		for(j = 0, f(l) = 0, a(l) = 0, lowdiff = inSize_; j < npartials; j++)
	  {
		  diff = abs(peaks_(j)-pit(0)*i);
			if(diff < lowdiff && diff/(pit(0)*i) < .05)
			{
				f(l) = peaks_(j);
				a(l) = log(pow(10,amps_(j)/20.0));
			}
		}
		if(lowdiff/(pit(0)*i) > .05)
		{
			for(f(l) = (int)(pit(0)*i), k = f(l)-5; k < (int)(pit(0)*i) + 5; k++)
				if(spectrum_(k) > spectrum_(k-1) && spectrum_(k) > spectrum_(k+1) && spectrum_(f(l)) <= spectrum_(k)) f(l) = k;
			k = f(l);
			if(spectrum_(k) > spectrum_(k-1) && spectrum_(k) > spectrum_(k+1) && spectrum_(f(l)) <= spectrum_(k))
			{				    
				diff = 0.5 * (spectrum_(k-1)-spectrum_(k+1))/(spectrum_(k-1)-2*spectrum_(k)+spectrum_(k+1));
				f(l) = k + diff;
				a(l) = log(spectrum_(k) - .25 * (spectrum_(k-1) - spectrum_(k+1)) * diff);
			}
			else a(l) = log(spectrum_(f(l)));
		}
    f(l) *= 2 * PI / inSize_;
		if(i > 10) if(round((f(l)-f(l-1))*inSize_/(2*pi*pit(0))) > 1) i = rpartials;
		if(a(l) <= hamsum2_ * get(noise_, f(l)) / (2 * hamsum_)) l--;
	}
	rpartials = l;
	
	fvec temp(maxorder_);
	wls(rpartials, maxorder_, f, a, h, temp, 0.6);
  for(i = 0; i < rpartials; i++) h(i) = get(temp, f(i))*hamsum_*hamsum_/(4*hamsum2_*get(noise_, f(i)));
	wls(rpartials, maxorder_, f, a, h, temp, 0.6);
	out(0) = pit(0);
	out(1) = pit(1);
	for(i = 0; i < maxorder_; i++) out(i+2) = temp(i);
	
/*	
  ofstream as("a",std::ios_base::app), fs("fs",std::ios_base::app), spec("spec",std::ios_base::app), env("env",std::ios_base::app), noise("noise",std::ios_base::app);			
  for(i = 0; i < 20; i++)
  {
    as << a(i) << endl;
	  fs << f(i)*inSize_/(2*PI) << endl;
  }
	for(i = 1; i < 1024; i++)
	{
	  spec << log(spectrum_(i)) << endl;
	  env << get(out, i*2*PI/inSize_) << endl;
		noise << hamsum2_*get(noise_, i*2*PI/inSize_)/(2*hamsum_) << endl;
	}*/
}


//solve equation Ac = b where A is factored to LDL* = b
void
DFCC::wls(int partials, int order, fvec& freq, fvec& amp, fvec& weight, fvec& o, float g)
{
  int i, j, k, sum;

  M.setval(0);
	for(i = 0; i < partials; i++) M(i, 0) = 1;
	for(i = 0; i < partials; i++)
	  for(j = 1; j < order; j++) M(i, j) = 2*cos(freq(i)*j);	

  fmatrix temp(order, order), temp1(order, order);
	for(i = 0; i < order; i++)
	  for(j = 0; j <= i; j++)
		  for(k = 0; k < partials; k++) temp(i, j) = temp(j, i) += weight(k)*M(k, i)*M(k, j); 
	for(i = 0; i < order; i++) temp(i, i) += 2*i*i*g;
	
	b.setval(0);
	for(i = 0; i < order; i++)
	  for(j = 0; j < partials; j++) b(i) += weight(j) * amp(j) * M(j, i);

	temp.invert(temp1);
	o.setval(0);
	for(i = 0; i < order; i++)
	 for(j=0; j<order; j++) o(i)+= temp(i, j)*b(j);
}










