/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/


/**
   \class Pitches
   \brief FFT based Pitches features 


   Calculate FFT mangitude based features. 
The Centroid, Rolloff and Flux are calculated 
on the magnitude spectrum of the input fvec 
and Zcrs are calculated in time domain. 
This System feature are typically used in classification. 
*/


#include "Dist.h"

Dist::Dist()
{
  inSize_ = DEFAULT_WIN_SIZE;
  init();
}

Dist::Dist(unsigned int inSize)
{
  inSize_ = inSize;
  init();
}


void 
Dist::init()
{
  bands_ = (int)(log10((float)inSize_/2)/log10(2.0)) - 4;
  outSize_ = bands_;
  featSize_ = outSize_;
  mags_.create(inSize_ + 2);
  featSize_=outSize_;
  fft_ = new FFT(inSize_, 1, 1, 22050);
  for (int i=0; i < outSize_; i++) featNames_.push_back("dist");
}



Dist::~Dist()
{
  delete fft_;
}


void 
Dist::process(fvec& in, fvec& out) 
{
  unsigned int i, j, k, unit = 16;
  float total = 0;
  assert((in.size() == inSize_) && (out.size() == outSize_));      
  
  fft_->process(in, mags_);

  for (i=0, j = 16; i < bands_; i++)
  {
    out(i) = 0;
    for(k = 0; k < unit ; k++, j+=2)
       out(i) += pow(mags_(j), 2);
    total += out(i);
    unit *= 2;
  }
  if(total!=0) for(i = 0; i<bands_; i++) out(i) /= total;
}




	

	
