/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/**
   \class Pitches
   \brief FFT based Pitches features 


   Calculate FFT mangitude based features. 
The Centroid, Rolloff and Flux are calculated 
on the magnitude spectrum of the input fvec 
and Zcrs are calculated in time domain. 
This System feature are typically used in classification. 
*/

#if !defined(__Dist_h)
#define __Dist_h

#include "FFT.h"

class Dist: public System
{
private:
  FFT *fft_;
  
  unsigned int bands_;
  
  vector<string> featNames_;
  fvec mags_;
  
  
public:
  Dist();
  Dist(unsigned int inSize);
  ~Dist();
  void init();
  void process(fvec& in, fvec& out);
};

#endif

	
