/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/



#if !defined(__Extractor_h)
#define __Extractor_h

#include "MSignal.h"
#include "FeatMatrix.h"

#include <string.h>

/**
   \class  Extractor
   \brief  Interface for extractors.

   Abstract base class used as interface for extractors. Extractors
   iterate over a MSignal applying a System possibly extracting feature
   values and optionally storing them to a FeatMatrix.  
*/


class Extractor
{
protected:
  string type_;
  MSignal *src_;
 
public:
  /** 
   * Initialize the extractor with a signal source given by src
   \param src : Iterate over signal by calling MSignal::get 
   */
  Extractor(MSignal* src);
  virtual ~Extractor();

  /** 
   * Iterate through all the signal 
   * equivalent to iterate(0, MSignal::iterations() )
   */
  void extract();
  void extract(FeatMatrix& res);


  
  /** 
   * Iterate through part of the signal specified by \c start and \c end 
   * The actual time corresponding to start and end 
   * are a function of the MSignal::srate_, MSignal::winSize_, 
   * MSignal::hopSize_ etc
   * \param start: starting window index for iteration
   * \param end: ending window index for iteration 
   */
  virtual void extract(unsigned int start, unsigned int end) =0;

  /** 
   * Iterate through part of the signal specified by \c start and \c end 
   * The actual time corresponding to start and end 
   * are a function of the MSignal::srate_, MSignal::winSize_, 
   * MSignal::hopSize_ etc. The calculated feature values 
   * are stored int fmatrix
   * \param start: starting window index for iteration
   * \param end: ending window index for iteration 
   * \param fmatrix: resulting FeatMatrix 
   */
  virtual void extract(unsigned int start, unsigned int end, FeatMatrix& fmatrix) = 0;
  
};


#endif	

