/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/** 
    \class GMM
    \brief GMM classifier
 
    GMM (Gaussian Mixture Model) classifier is a classifier where each
class is modelled as a mixture of multidimensional Gaussian
distribution. The features are assumed to be uncorrelated and a
diagonal covariance matrix is used.  */


#if !defined(__GMM_h)
#define __GMM_h


#include <stdio.h>

#include <stdlib.h>
#include <math.h>
#include <string.h>
#include <iostream.h>
using namespace std;

#include "FeatMatrix.h"
#include "Classifier.h"
#include "fmatrix.h"
#include "defs.h"
 
class GMM:public Classifier
{
private:
  unsigned int k_;
  unsigned int iterations_;		// EM algorithm iterations
  unsigned int kiterations_;		// K-MEANS initialization iterations
  unsigned int eiterations_;		// Evaluation-training iterations
   
  
  float factor_;
  fvec classSizes_;
  fvec sum_;
  
  fvec accumrow_;
  fvec row_;
  fvec temp_;
  
  fmatrix sprobs_;
  fmatrix likelihoods_;
  
  vector<fmatrix> probs_;
  vector<fmatrix> ssprobs_;
  
  vector<fmatrix> vars_;
  vector<fmatrix> covars_;
  vector<fmatrix> means_;
  vector<fmatrix> omeans_;
  vector<fvec> weights_;
  
  void initialize();
  void update();
  double gaussian(unsigned int cl, unsigned int k, fvec& vec);
  
public:
  GMM();
  GMM(unsigned int k_);
  ~GMM();
  void hardwire();
  void train(FeatMatrix* fmap, unsigned int lcol);	// Train the classifier 
  unsigned int classify(fvec& vec);
  void mask(ivec& vec);
  void read(string filename);
  void write(string filename);
  void info();
  float evaluate(FeatMatrix& featMatrix);
  friend ostream& operator<<(ostream&, GMM&);
  friend istream& operator>>(istream&, GMM&);
};




 
#endif


	

	
