/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/** 
    \class Gain
    \brief Multiply input fvec with gain

   Simple System example. Just multiply the values of the input fvec
with gain and put them in the output vector. 
*/



#include "Gain.h"

Gain::Gain()
{
  inSize_ = DEFAULT_WIN_SIZE;
  outSize_ = DEFAULT_WIN_SIZE;
  gain_ = 0.5;
}

Gain::~Gain()
{
}


Gain::Gain(unsigned int inSize, float gain)
{
  inSize_ = inSize;
  outSize_ = inSize;
  gain_ = gain;
}


void 
Gain::process(fvec& in, fvec& out) 
{
  assert((in.size() == inSize_) && (out.size() == outSize_));  
  for (unsigned int i=0; i < in.size(); i++)
    {
      out(i) = gain_ * in(i);
    }
}


	
