/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/ 






/**
   \class Gmpg
   \brief MP3 decoder.
   
   Decoder for mp3 files. For now it works with MPEG 2 layer III files
   (mp3 files) (ISO 13818-3 MPEG-2 audio decoder. It is based on
   various open source software available including the reference
   software implementation by Fraunhofer institute. I wrote this software
   for my own research purposes because all the implementations 
   I could find where either too slow or completely unreadable.  
*/


#if !defined(__Gmpg_h)
#define __Gmpg_h


#include <stdio.h> 
#include "MSignal.h"
#include "defs.h" 
#include "SoundFileIO.h"
#include <math.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>


#include "Gmpg_main.h"			// main decoder structures
#include "Gmpg_huffman.h"		// huffman tables

#include <string.h> 
using namespace std;

// #define DECODE
#define MEMORY



class Gmpg
{
private:
  string filename_;
  float  *buffer_;
  int samples_;
  int frames_;
  FILE* fp_;

  /* Fixed size buffers used for circular list of 
     read buffers. Layer III uses bit-reservoir technique 
     where data corresponding to a frame in time can 
     span multiple previous frames. */
  unsigned char g_data_buffers[MAX_READ_LIST][MAX_FRAME_SIZE];
  unsigned char g_decode_buffer[MAX_FRAME_SIZE];
  
  /* Alias reduction tables */
  double alias_coefs[8];
  int pretab[22];
  
  /* Global static buffers used for decoding */
  double g_N[64][32];
  double g_V[1024];


  double old_mean_sb[32];
  double v_buf[2][1024];
  int v_offset[2];


  
public:
  Gmpg(string filename, FILE *fp);
  ~Gmpg();
  void go();
  void init(MPEG_decoder *decptr);
  void init_read_list(MPEG_decoder *decptr);
  unsigned int get_bits(Bs_ptr *bs_prt, int num_bits);
  void decoder_info(MPEG_decoder* decptr);
  void check_header(Header *hdr);
  void write_au_header(int length, int channels, int sample_rate);
  int read_header(FILE *fp, MPEG_decoder *decptr);
  int read_frame_data(FILE *fp, MPEG_decoder *decptr);
  void read_side_info(MPEG_decoder *decptr);
  void read_compressed(MPEG_decoder *decptr);
  void decode_huffman(MPEG_decoder *decptr,int ch, int huffman_bits, 
		      int part2_bits);
  void dequantize(MPEG_decoder *decptr, int ch);
  void reorder(MPEG_decoder *decptr, int ch);
  void joint_stereo(MPEG_decoder *decptr);
  void alias_reduce(MPEG_decoder *decptr, int ch);
  void window(int block_type, double x[36], double z[36]);
  void imdct(MPEG_decoder *decptr, double *in, double *out, int block_type);
  void hybrid(MPEG_decoder *decptr, int ch);
  void synthesis(MPEG_decoder *decptr, int ch);
  void decode(MPEG_decoder *decptr);


  void get(fvec& win, unsigned int long index);
  unsigned int getSize();
  
};




#endif


	


