/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/


/**
   \class Instrument
   \brief Interface for instruments
   
   Abstract base class for instruments. 
Specific instruments like Sine, Wavetable etc, are 
children of this class. 
*/




#include "MSignal.h"


#if !defined(__Instrument_h)
#define __Instrument_h



class Instrument: public MSignal 
{  
protected:
  float pitch_table_[128];        			// pitch table
public:
  Instrument(unsigned int srate);
  virtual void init(float freq, float amp, float duration) =0;
  ~Instrument();
  virtual void get(fvec &win, unsigned long index) = 0;
  virtual void put(fvec &win, unsigned long index) = 0;
};


#endif	
