/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/


/** 
    \class KNN.h 
    \brief K nearest neighbors classifier 

    The K nearest neighbors classifier (KNN) is a non-parametric 
classifier where each point is classified according to the majority 
class label vote of its k nearest neighbors. 

*/

#if !defined(__KNN_h) 
#define __KNN_h

#include "Classifier.h"

#define MAX_INT 100000

class KNN:public Classifier 
{
private: 
  FeatMatrix trainMap_;
  unsigned int k_;
  ivec votes_;
  fvec mins_;
  ivec minsPos_;
  unsigned int rows_;
  unsigned int cols_;
  fvec vars_;
  fvec covars_;
  
public:
  KNN(unsigned int k);
  ~KNN();
  void train(FeatMatrix* featMatrix, unsigned int lcol);
  unsigned int classify(fvec& vec);
};


#endif 
