/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/** 
    \class Median
    \brief Median of input fvec

    Median of the input fvec.
*/


#include "Median.h"

Median::Median()
{
  inSize_ = DEFAULT_WIN_SIZE;
  outSize_ = 1;
  temp_.create(inSize_);
}

Median::Median(unsigned int inSize)
{
  inSize_ = inSize;
  outSize_ = 1;
  temp_.create(inSize_);
}


int 
Median::partition(float a[], int l, int r)
{
  int i= l-1;
  int j = r;
  float v = a[r];
  float temp;
  
  for (;;) 
    {
      // move pointers
      while(a[++i] <v);
      while(v < a[--j]) if (j ==l) break;
      if (i >= j) break;
      
      // swap elements
      temp = a[i];
      a[i] = a[j];
      a[j] = temp;
    }
  
  // Swap elements
  temp = a[i];
  a[i] = a[r];
  a[r] = temp;
  return i;
}




void 
Median::process(fvec& in, fvec& out) 
{
  assert((in.size() == inSize_) && (out.size() == outSize_));  
  // Median calculatino based on quicksort partitioning

  int l = 0;
  int r = inSize_ -1;
  int k = inSize_/2;
  int i;

  temp_ = in;
  float *windata = temp_.getData();
  float median;
  
  while (r > l) 
    {
      i = partition(windata, l, r);
      if (i>=k) r = i-1;
      if (i<=k) l = i+1;
    }
  
  median = windata[k];
  out(0) = median;
}


	
