/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/** 
    \class MemMFCC
    \brief Memory MFCC statistics

   Compute running statistics (mean,variance) of Mel-frequency
cepstral coefficients (MFCC) using a 1-second Memory. MFCC are
frequently used in speech recognition research.  
*/



#include "MemMFCC.h"

MemMFCC::MemMFCC()
{
  inSize_ = DEFAULT_WIN_SIZE;
  zeroSize_ = 0;
  outSize_ = 10;
  memory_ = NULL;
  mfcc_ = NULL;
  init();
}

MemMFCC::MemMFCC(MSignal* src)
{
  inSize_ = src->winSize();
  zeroSize_ = src->zeroSize();
  outSize_ = 10;
  memory_ = NULL;
  mfcc_ = NULL;
  init();
}

MemMFCC::~MemMFCC()
{
  delete memory_;
  delete mfcc_;
}


void 
MemMFCC::init()
{
  int memSize = 40;  
  memory_ = new Memory(outSize_ /2, memSize);
  mfcc_ = new MFCC(inSize_, zeroSize_);
  temp1_.create(13);
  temp2_.create(5);
  
  featSize_ = 10;
  featNames_.push_back("mMFCC00");
  featNames_.push_back("mMFCC01");
  featNames_.push_back("mMFCC02");
  featNames_.push_back("mMFCC03");
  featNames_.push_back("mMFCC04");
  featNames_.push_back("vMFCC00");
  featNames_.push_back("vMFCC01");
  featNames_.push_back("vMFCC02");
  featNames_.push_back("vMFCC03");
  featNames_.push_back("vMFCC04");
  
}




void 
MemMFCC::process(fvec& in, fvec& out)
{
  unsigned int i;
  if ((in.size() != inSize_) || (out.size() != outSize_))
    {
      cerr << "Warnging: ShapeDesc::process:  inSize_ and input window size do not agree" << endl;
      return;
    }  
  mfcc_->process(in, temp1_);
  for (i=0; i< 5; i++)
    temp2_(i) = temp1_(i);
  memory_->process(temp2_, out);
}

