/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/** 
    \class MemMFCC
    \brief Memory MFCC statistics

   Compute running statistics (mean,variance) of Mel-frequency
cepstral coefficients (MFCC) using a 1-second Memory. MFCC are
frequently used in speech recognition research.  
*/

#if !defined(__MemMFCC_h)
#define __MemMFCC_h

#include "System.h"
#include "MagFFT.h"
#include "fvec.h"
#include "fmatrix.h"
#include <math.h>
#include "Hamming.h"
#include "MFCC.h"
#include "Memory.h"
#include "MSignal.h"

class MemMFCC: public System
{
private:
  Memory* memory_;
  MFCC* mfcc_;
  unsigned int zeroSize_;
  fvec temp1_;
  fvec temp2_;
  
  
public:
  MemMFCC();
  MemMFCC(MSignal *src);
  ~MemMFCC();
  void init();
  void process(fvec& in, fvec& out);
};


#endif 
