/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/** 
    \class Memory
    \brief Memory for running fvec statistics

   Compute running statistics (mean,variance) of the input 
fvec using a circular-buffer memory.
*/




#if !defined(__Memory_h)
#define __Memory_h

#include "System.h"	
#include "NormRMS.h"
#include "fmatrix.h"

class Memory: public System
{
private:
  fmatrix past_;
  fvec meanRow_;
  fvec stdRow_;
  unsigned int end_;
  unsigned int memSize_;
  
public:
  Memory();
  Memory(unsigned int inSize, unsigned int memSize);
  ~Memory();
  void process(fvec& in, fvec& out);
  void getPerc(float perc, fvec& out);
  
};

#endif

	
