/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/**
   \class  MultiPitchExtractor
   \brief  Single Vector feature extractor for beat information

   Feature vector of beat related information calculated using the DWT. 
*/





#if !defined(__MultiPitchExtractor_h)
#define __MultiPitchExtractor_h

#include "Extractor.h"
#include "MultiPitchExtractor.h"
#include "Hamming.h"
#include "MagFFT.h"
#include "Centroid.h"
#include "DB.h"
#include "Series.h"
#include "RMS.h"
#include "WaveletPyramid.h"
#include "Daub4.h"
#include "FullWaveRectifier.h"
#include "OnePole.h" 
#include "Norm.h"
#include "AutoCorrelationFFT.h" 
#include "DownSampler.h" 
#include "PeakPicker.h" 
#include "NormRMS.h"
#include "MultiPitch.h"
#include "PitchHistoFeatures.h"
#include "pitchfind.h"

class MultiPitchExtractor:public Extractor
{
 private:
  MultiPitch* multipitch_; 

  unsigned int featSize_;
  vector<string> featNames_;
  fvec esacf_;
  fvec res_;
  fvec in_;
  fvec peaks_; 
  PeakPicker *picker_;  
  fvec histo_;
  fvec fhisto_;
  
  
 public:
  MultiPitchExtractor(MSignal* in);
  ~MultiPitchExtractor();
  void extract();
  void extract(unsigned int start, unsigned int end);
  void extract(unsigned int start, unsigned int end, FeatMatrix& fmatrix);
  unsigned int outSize();
};


#endif	


	
