/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/


/** 
    \class MultiPitchFeature
    \brief Features for running pitch histogram
*/




#include "MultiPitchFeature.h"

MultiPitchFeature::MultiPitchFeature()
{
  inSize_ = DEFAULT_WIN_SIZE;
  init();
  outSize_ = series_->outSize();
}

MultiPitchFeature::~MultiPitchFeature()
{
  delete multipitch_;
  delete peakpicker_;
  delete memory_;
  delete series_;
}


MultiPitchFeature::MultiPitchFeature(MSignal *src)
{
  inSize_ = src->winSize();
  init();  
  outSize_ = series_->outSize();
}

void
MultiPitchFeature::init()
{
  multipitch_ = new MultiPitch(inSize_);  
  peakpicker_ = new PeakPicker(inSize_, 1, 3, 22, inSize_, 50);
  memory_ = new Memory(peakpicker_->outSize(), 40);
  vector<System *> featArray;
  featArray.push_back(multipitch_);
  featArray.push_back(peakpicker_);
  featArray.push_back(memory_);
  series_ = new Series(featArray);

  featSize_ = series_->outSize();
  
  for (unsigned int i=0; i < featSize_; i++)
    featNames_.push_back("MPitchFeature");

}

void 
MultiPitchFeature::process(fvec& in, fvec& out) 
{
  assert((in.size() == inSize_) && (out.size() == outSize_));  
  series_->process(in, out);
}

	
