/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/


/**
   \class  NNShuffler
   \brief  Interface for segmentors

   Abstract base class used as interface for segmentors. NNShufflers 
   as input a FeatMatrix and produce a temporal segmentation 
   of the FeatMatrix into regions in time stored as 1.0 and 0.0 
   in a fvec. 
*/


#include "NNShuffler.h"

NNShuffler::NNShuffler()
{
}


NNShuffler::~NNShuffler()
{
}


void 
NNShuffler::shuffle(FeatMatrix& matrix, ivec &mapping)
{
  float dist;
  float min;
  unsigned int minIndex =0;
  int r,s,c;
  unsigned int cols;
  cols = matrix.cols();
  unsigned int size;
  size = mapping.size();

  fvec vars(cols);
  fvec covars(cols);
  vars = matrix.varRow();
  fvec v1(cols);
  fvec v2(cols);

  for (c=0; c < (int)cols; c++)
    {
      if (vars(c) <= 0.000001) 
	covars(c) = 1.0;		// No variance
      else
	covars(c) = 1.0 / vars(c);
    }
  

  for (r=0; r < (int)size; r++)
    {
      min = 100000000;
      for (s=0; s < (int)size; s++)
	{
	  v1 = matrix.row(r);
	  v2 = matrix.row(s);
	  dist = mahalanobis(v1, v2, covars);
	  if (dist < min)
	    {
	      if ((r < s -5) || ( r > s + 5))
		{
		  min = dist;
		  minIndex = s;
		}
	    }
	}
      mapping(r) = minIndex;
    }
}
