/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/** 
    \class PeakEnhancer
    \brief Enhance peaks that are harmonic
    
    Enhance harmonic peaks resulting from autocorrelation periodicity 
detection by applying half wave rectification and then subtracting the 
upsampled fvec from the original fvec. 
*/



#include "PeakEnhancer.h"
#include "MSignal.h"
#include "HalfWaveRectifier.h"
#include "fvec.h"
#include "Filter.h"  


PeakEnhancer::PeakEnhancer()
{
  inSize_ = DEFAULT_WIN_SIZE;
  outSize_ = DEFAULT_WIN_SIZE;
  itnum_ = DEFAULT_ITNUM;
}

PeakEnhancer::PeakEnhancer(unsigned int inSize, unsigned int itnum)
{
  inSize_ = inSize;
  outSize_ = inSize;
  itnum_ = itnum;
}

PeakEnhancer::~PeakEnhancer()
{
}


void 
PeakEnhancer::process(fvec& in, fvec& out)
{
  assert((in.size() == inSize_) && (out.size() == outSize_));      
  int i;
  fvec num_low(3);
  fvec denom_low(3);
  num_low(0)=0.1207; num_low(1)=0.2415; num_low(2)=0.1207;
  denom_low(0)=1.0; denom_low(1)=-0.8058; denom_low(2)=0.2888;

  fvec temp(inSize_);
  fvec temp2(inSize_);
  HalfWaveRectifier rect(inSize_); 
  Filter lowpass(num_low, denom_low, "lpf");
  
  rect.process(in, temp);
  temp2.setval(0, inSize_-1, 0.0);
  for (i=0; i<inSize_/itnum_; i++) temp2(itnum_*i)=temp(i);
  lowpass.process(temp2, temp2);  

  in=in-temp2;
  out=in; 
}

