/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/** 
    \class PeakPicker
    \brief Pick Peaks 

   Simple Peak Picking System 
*/


#include "PeakPicker.h"

PeakPicker::PeakPicker()
{
  inSize_ = DEFAULT_WIN_SIZE;
  temp_.create(DEFAULT_WIN_SIZE);
  temp2_.create(DEFAULT_WIN_SIZE);
  histo_.create(300);
  factor_ = 1;
  numPeaks_ = 3;
  outSize_ = 2 * numPeaks_;
  
  start_ = 300;  //beat case
  stop_ = 3200;  //beat case
  bw_ = 100;     //neighborhood to zero out (100 is for beat case)  
}

/*
PeakPicker::~PeakPicker()
{
  //  delete memory_;
}
*/

PeakPicker::PeakPicker(unsigned int inSize, unsigned int factor,unsigned int numPeaks, unsigned int start, unsigned int stop, int bw) {
  inSize_ = inSize / factor;
  temp_.create(inSize_);
  temp2_.create(inSize_);
  histo_.create(300);
  factor_ = factor;
  numPeaks_ = numPeaks;
  outSize_ = 2 * numPeaks_;  
  start_ = start;
  stop_ = stop;
  bw_ = bw;
}





void 
PeakPicker::process(fvec& in, fvec& out) 
{
  assert((in.size() == inSize_) && (out.size() == outSize_));  

  unsigned int k,i;
  fvec maximums;
  ivec maxPositions;
  maximums.create(numPeaks_);
  float max =0.0;
  
  maxPositions.create(numPeaks_);
  maxPositions.setval(0);
  int j;

  for (k=0; k < numPeaks_; k++)
    {
      max = 0.0;
      for (i= start_; i < stop_-1; i++)
	{
	  if ((in(i) > max) && (in(i) > in(i-1)) && (in(i) > in(i+1)))
	    {
	      max = in(i);
	      maximums(k) = max;
	      maxPositions(k) = i;
	    }
	}
      if (maxPositions(k) >= bw_) j = maxPositions(k) - bw_;
      else j = 0; 
      for (; j < maxPositions(k) + bw_; j++)
	in(j) = 0.0;

    }
  
  for (k=0; k < numPeaks_; k++)
    {
      out(k) = maximums(k);
      out(k+numPeaks_) = maxPositions(k);
    }
  return;
}
