/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/**
   \class  PitchExtractor
   \brief  Single Vector feature extractor for Sound Effects

   This is an example of a specific extractor for calculating 
a single feature vector. It is used for feature calculation for sound effects 
where each file is represented by a single feature vector. 
*/


#include "PitchExtractor.h"

PitchExtractor::PitchExtractor(MSignal* in):Extractor(in)
{
  unsigned int inSize = in->winSize();
  
  in_.create(inSize);
  out_.create(1);
  res_.create(1);
  
  amdf_ = new AMDF(inSize);
  vpick_ = new ValleyPick(inSize);
  
  vector<System *> featArray;
  featArray.push_back(amdf_);
  featArray.push_back(vpick_);
  series_ = new Series(featArray);

  featSize_ = 1;
  featNames_.push_back("MedianPitch");
}

PitchExtractor::~PitchExtractor()
{
  delete amdf_;
  delete vpick_;
  delete series_;
  delete median_;
}

unsigned int 
PitchExtractor::outSize() 
{
  return featSize_;
}


void
PitchExtractor::extract()
{
  extract(0, src_->iterations());
}

void 
PitchExtractor::extract(unsigned int start, unsigned int end, 
		      FeatMatrix& fmatrix) 
{
  int load_matrix = 1;
  assert((fmatrix.rows() == 1) && (fmatrix.cols() == res_.size()));
  pitchVec_.create(end-start);
  unsigned int t;
  unsigned int p=0;
  fmatrix.setFeatNames(featNames_);
  
  median_ = new Median(end-start);
  
  
  for (t=start; t< end; t++)
    {
      src_->get(in_, t);
      series_->process(in_, out_);
      pitchVec_(t-start) = out_(0);
      if (t % ((end-start)/5) == 0)
	{
	  cerr << p * 20  << "% ";
	  p++;
	}      
      cout << out_(0) << endl;
    } 
  median_->process(pitchVec_, res_);
  cout << "Median pitch = " << res_(0) << endl;
  if (load_matrix) 
    {
      //res_(0) = pitchVec_.mean();
      fmatrix.row(0, res_);
    }
  
  cerr << "\nFeature Extractor done" << endl;
}


void
PitchExtractor::extract(unsigned int start, unsigned int end)
{
  unsigned int t;
  for (t=start; t< end; t++)
    {
      src_->get(in_, t);
      if (t % ((end-start)/10))
	{
	  cerr << (end-start)/t * 100.0 << "% processed" << endl;
	}      
    }
}

