/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/**
   \class Pitches
   \brief FFT based Pitches features 


   Calculate FFT mangitude based features. 
The Centroid, Rolloff and Flux are calculated 
on the magnitude spectrum of the input fvec 
and Zcrs are calculated in time domain. 
This System feature are typically used in classification. 
*/

#if !defined(__Pitches_h)
#define __Pitches_h

#include "MultiPitchExtractor.h"
#include "Series.h"
#include "RMS.h"
#include "PeakPicker.h" 
#include "MultiPitch.h"
#include "System.h"
#include "MagDB.h"
#include "math.h"
#include "Hamming.h"


class Pitches: public System
{
private:
  unsigned int zeroSize_;
  Series* series_;
  Hamming* hamming_;
  MagFFT* magfft_;

  int pitches_;
  
  vector<string> featNames_;
  fvec esacf_;
  fvec peaks_;
  int bins, start, N, *binStart, *binSz;
  float *binFreq, *binWidth, rate, size, maxsum;
  float **Xe, **Lb;
  float *weights, *max_smooth, *smooth;
  int *candidates;
  int *bands;
  int ***curves;
  int **curve;
  int round(float r);
  float freq(int n);
  int index(float f);
  float triangulate(int n, int Kb, int mid);
  void getPitch(int pitch, fvec& out);
  void printFfts();
  void printWeights();
  float estimate(int n, int band, float weight, float inharm, int last_partial, int last_freq, int last_gap);
  ofstream ffts, lbs, smoothed;
  float offset, lboffset;
  
public:
  Pitches();
  Pitches(int inSize, int pitches);
  ~Pitches();
  void init();
  void process(fvec& in, fvec& out);
};

#endif

	
