/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/**
   \class  SfxExtractor
   \brief  Single Vector feature extractor for Sound Effects

   This is an example of a specific extractor for calculating 
a single feature vector. It is used for feature calculation for sound effects 
where each file is represented by a single feature vector. *
*/


/* SfxExtractor:
   Iterate through a sound object and plot each window
*/ 

#if !defined(__SfxExtractor_h)
#define __SfxExtractor_h

#include "Extractor.h"
#include "SfxExtractor.h"
#include "Hamming.h"
#include "MagFFT.h"
#include "Centroid.h"
#include "DB.h"
#include "Series.h"
#include "RMS.h"

class SfxExtractor:public Extractor
{
protected:

  fvec in_;
  fvec out_;
  fvec res_;
  
  fvec rmsVec_;
  fvec centroidVec_;
  fvec rmsRes_;
  fvec centroidRes_;
  
  Hamming* hamming_;
  MagFFT* magfft_;
  Centroid* centroid_;
  Series* series_;
  RMS* rms_;
  

  unsigned int featSize_;
  vector<string> featNames_;
  
  
 
public:
  SfxExtractor(MSignal* in);
  ~SfxExtractor();
  void extract();
  void extract(unsigned int start, unsigned int end);
  void extract(unsigned int start, unsigned int end, FeatMatrix& fmatrix);
  unsigned int outSize();
};


#endif	


	
