/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/**
   \class  SvFeatExtractor
   \brief  Single Vector Feature calculation extractor

   This extractor is initialized with a MSignal and a System that calculate 
feature values given a window of data. The extractor iterates through the given MSignal applying the System to each window of data and possibly storing their resulting feature vectors to a FeatMatrix with 
a single row.
*/


#if !defined(__SvFeatExtractor_h)
#define __SvFeatExtractor_h

#include "Extractor.h"
#include "SvFeatExtractor.h"
#include "Hamming.h"
#include "MagFFT.h"
#include "DB.h"

/**
   \class  SvFeatExtractor
   \author George Tzanetakis 
   \version Marsyas 0.1
   \brief  Interface for extractors.
   
   Iterates over a MSignal calculating feature values 
   based on a specific feature given as an argument to the constructor. 
   Optional plotting is supported controlled by mode_
*/

class SvFeatExtractor:public Extractor
{
protected:
  string type_;
  System *feat_;
  fvec in_;
  fvec out_;
  fvec res_;
  int mode_;

 
public:
  SvFeatExtractor(MSignal* in, System* feat);
  ~SvFeatExtractor();
  unsigned int outSize();
  void extract(unsigned int start, unsigned int end);
  void extract(unsigned int start, unsigned int end, FeatMatrix& fmatrix);
};


#endif	


	
