/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/** 
    \class Talk
    \brief Message handler

    Reacts to various messages using Communicator. 
Specifically used for handling the graphical user interface events. 
*/





#if !defined(__Talk_h)
#define __Talk_h


#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include "Communicator.h"
#include "fvec.h"
#include "Util.h"
#include "MSignal.h"
#include "PlotExtractor.h"
#include "MagFFT.h"
#include "Gaussian.h"
#include "Spectral.h"
#include "FeatExtractor.h"
#include "MemMFCC.h"
#include "SpectralSegm.h"
#include "SfxExtractor.h"
#include "TimeLine.h"
#include "SegmentorSortedPeaks.h"
#include "ColorPlot.h"
#include "MagDB.h"

#include <iostream.h> 
#include <map>
#include <string.h>
using namespace std;


#define SEPCHARS " \t\n"
#define UNSPECIFIED -1
#define WIN_SIZE 512

class Talk
{
private:
  Communicator *communicator_;
  void cmd_load(string fname, unsigned int lineSize);
  void cmd_classify(string system, string classifier, unsigned int start, unsigned int end, unsigned int winSize);
  void cmd_segment(string systemName, unsigned int memSize, unsigned int numPeaks, unsigned int peakSpacing, unsigned int start, unsigned int end, unsigned int winSize);
  
  void cmd_fullfft(unsigned int winSize);
  void cmd_fft(unsigned int start, unsigned int winSize);
  void cmd_colorgram(string collection, string system, unsigned int start, unsigned int end, unsigned int win_size);
  MSignal *src_;
  
public:
  Talk();
  ~Talk();
  void init(Communicator* communicator);
  void process(char *message);
};




#endif


	

	
