/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/** 
    \class WaveletStep
    \brief One step of Discrete Wavelet Transform (DWT)

   One step of the Discrete Wavelet Transform (DWT) 
pyramid algorithm. Called by WaveletPyramid. The code is 
is based on the Numerical Recipies wavelet code. 
*/





#include "WaveletStep.h"

WaveletStep::WaveletStep()
{
  inSize_ = DEFAULT_WIN_SIZE;
  outSize_ = DEFAULT_WIN_SIZE;
  direction_ = WVL_FORWARD;
}

WaveletStep::WaveletStep(unsigned int inSize, int direction)
{
  inSize_ = inSize;
  outSize_ = inSize;
  direction_ = direction;
}


void 
WaveletStep::setSize(unsigned int processSize) 
{
  processSize_ = processSize;
}


void 
WaveletStep::process(fvec& in, fvec& out) 
{
  assert((in.size() == inSize_) && (out.size() == outSize_));  
  for (unsigned int i=0; i < in.size(); i++)
    {
      out(i) = in(i);
    }
}


	
