/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/


/** 
    \class Zcrs 
    \brief Zcrs of the input fvec

    Calculate the Zcrs of the input fvec in 
the time domain. 
*/


#include "Zcrs.h"

Zcrs::Zcrs()
{
  inSize_ = DEFAULT_WIN_SIZE;
  outSize_ = 1;
}

Zcrs::Zcrs(unsigned int inSize)
{
  inSize_ = inSize;
  outSize_ = 1;
}


Zcrs::~Zcrs()
{
}


void 
Zcrs::process(fvec& in, fvec& out) 
{
  if ((in.size() != inSize_) || (out.size() != outSize_))
    {
      cerr << "Warning: Zcrs::process : inSize_ and input window size do not agree" << endl;
      return;
    } 
  float zcrs = 1.0;
  for (unsigned int i=1; i<inSize_; i++)
    {
      if (((in(i-1) > 0) && (in(i)<0)) ||
	  ((in(i-1) < 0) && (in(i)>0)))
	zcrs ++;
    }
  out(0) = zcrs;
}


	

	
