/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/* sfplay:
   Command line utility for playing soundfiles. 
*/

#if HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include "MSignal.h"
#include "SoundFileIO.h"
#include "defs.h"
#include "Util.h"

#ifndef WIN32
#include "Audio.h"
#else
#include "system/AudioDummy.h"
#endif 

void 
chop(float offset, float seconds, string filename)
{
  MSignal *src;
  Util util;
  src = util.sfopen(filename, MRS_SF_READ);
  if (src == NULL) 
    exit(1);
  src->initWindow(1024, 1024, 0, 0);
  fvec window(src->winSize());
  float iterations = (seconds * src->srate()) / src->winSize();
  unsigned int start = (unsigned int)((offset * src->srate()) / src->winSize());
  // unsigned int start = 0;
  if ((iterations >= src->iterations())||(seconds == -1.0))
    iterations = src->iterations();
  
  AuFileIO *dest;
  dest = new AuFileIO("chopped.au", MRS_SF_WRITE);
  dest->initMSignal(src->srate(), 1);
  dest->init();
  dest->initWindow(1024, 1024, 0, 0);
  for (unsigned int i=start; i< start+iterations; i++)
    {
      src->get(window,i);
      dest->put(window,i-start);
    }
  delete src;
}
	
int 
main(int argc, char **argv)
{
  if (argc == 1)
    {
      cerr << "Usage :" << argv[0] << " offset duration (-1 for full length) filenames\n";
      exit(1);
    }
  int i;
  float offset = atof(argv[1]);
  float seconds = atof(argv[2]);
  cerr << "Chop offset   = " << offset << endl;
  cerr << "Chop duration = " << seconds << endl;
  for (i=3; i < argc; i++)
    {
      cerr << "Chopping " << argv[i] << endl;
      chop(offset, seconds, argv[i]);
    }
  return 0;
}
