/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <stdio.h>
#include "FeatMatrix.h"
#include "MSignal.h"
#include "fmatrix.h"
#include "SoundFileIO.h"
#include "defs.h"
#include "Util.h"
#include "Gain.h"
#include "Hamming.h"
#include "MagFFT.h"
#include "FeatExtractor.h"
#include "Series.h"
#include "MagDB.h"
#include "Sine.h"
#include "Spectral.h"
#include "SpectralSegm.h"
#include "SfxExtractor.h"
#include "MemMFCC.h"
#include "LPC.h"
#include "MultiPitchFeature.h"
#include "DWTC.h"
#include "BeatExtractor.h"
#include "SvFeatExtractor.h"
#include "MultiPitchExtractor.h"
#include "GenreExtractor.h"
#include "ExtractorDictionary.h"
#include "FileName.h"

#include <map>
#include <string.h>
using namespace std;




void 
extract(string from, string to, string extractorstr)
{
  /* Prepare name (remove full path) for feature matrix */ 
  FileName fromFile(from);
  string fromName = fromFile.name();

  /* Prepare signal */ 
  MSignal *src;
  Util util;
  src = util.sfopen(from, MRS_SF_READ);
  src->initWindow(512, 512, 0, 0);
  
  /* Special window sizes for specific extractors */ 
  if (extractorstr == "LPC")
    src->initWindow(512, 128, 0, 0);
  if (extractorstr == "BEAT") 
    src->initWindow(65536, 65536/2,0,0);
  if (extractorstr == "DWTC")
    src->initWindow(16384, 512, 0, 0);
  if (extractorstr == "MPITCH")
    src->initWindow(512, 512, 0, 0);

  /* Extractor dictionary contains all the available 
     extractors for feature calculation */ 
  ExtractorDictionary dict(src);

  
  /* find appropriate extractor and feature matrix result */ 
  Extractor *extr = dict.getExtractor(extractorstr);
  FeatMatrix *featMatrix = dict.getFeatMatrix(extractorstr);
  
  /* do the feature calculation */
  extr->extract(*featMatrix);
  
  /* append the feature matrix to the output file for the collection */
  string name = fromName.substr(0, fromName.size()-3);
  featMatrix->setName(name);
  featMatrix->write(to);
  cout << "Wrote " << to << endl;
  delete src;
}



 

int
main(int argc, char **argv)
{
  ExtractorDictionary dict;
  
  if (argc != 3)
    {
      cerr << "Usage :" << argv[0] <<" extractor audiofile \n";
      cerr << "Supported extractors are: " << endl;
      cerr << dict << endl;    
      exit(1);
    }
  
  string extractorstr = argv[1];
  string from = argv[2];
  string to = from.substr(0, from.size()-3);
  to += extractorstr;
  extract(from,to,extractorstr);
  return 0;
}
