/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/** 
    \class imatrix
    \brief Matrix of integer values

    Matrix of integer values. Similar to ivec. Basic 
arithmetic operations and statistics are supported. 
*/


#if !defined(__imatrix_h)
#define __imatrix_h


#include <fstream.h>
#include <iostream.h>
#include <string.h>
using namespace std;



#include "fvec.h"
#include "ivec.h"

class imatrix
{
protected:
  int *data_;
  unsigned int size_;
  unsigned int rows_;
  unsigned int cols_;
  string name_;
  ivec row_;
  ivec col_;
  bool printHeader_;
  
public: 
  imatrix();
  ~imatrix();
  imatrix(unsigned int rows);
  imatrix(unsigned int rows, unsigned int cols);
  imatrix(const imatrix& a);
  imatrix& operator=(const imatrix& a);
  
  // Initialization functions
  void create(unsigned int rows, unsigned int cols);
  void create(unsigned int rows);
  void identity(unsigned int rows);
  void setval(int val);

  // Statistics 
  ivec  row(const unsigned int r);
  ivec  col(const unsigned int c);
  void row(const unsigned int r, ivec& vec);
  void col(const unsigned int r, ivec& vec);
  unsigned int rows();
  unsigned int cols();
  friend istream& operator>>(istream&, imatrix&);
  friend ostream& operator<<(ostream&, const imatrix&);
  imatrix& operator*=(const imatrix& matrix);
  imatrix& operator+=(const imatrix& matrix);
  imatrix& operator/=(const int val);
  imatrix& operator*=(const int val);
  
  // Misc
  void setName();
  void printHeader(bool flag);
  
  /* row referencing operator */
  
  int& operator()(const long r, const long c);
  int operator()(const long r, const long c) const;

  void read(string filename);
  void write(string filename);
  
  
};




inline 
int imatrix::operator()(const long r, const long c) const
{
  return data_[r * cols_ + c];
}


inline 
int& imatrix::operator()(const long r, const long c)
{
  return data_[r * cols_ + c];
}




#endif


