#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>
#include "FeatMatrix.h"
#include "BatchExtract.h"
#include "MSignal.h"
#include "fmatrix.h"
#include "SoundFileIO.h"
#include "defs.h"
#include "Util.h"
#include "Gain.h"
#include "Hamming.h"
#include "MagFFT.h"
#include "FeatExtractor.h"
#include "Series.h"
#include "MagDB.h"
#include "Sine.h"
#include "Spectral.h"
#include "SpectralSegm.h"
#include "FeatExtractor.h"
#include "Shuffler.h" 
#include "MFCC.h"
#include "ExtractorDictionary.h"
#include "Classifier.h"
#include "MemMFCC.h"
#include "LPC.h"
#include "SfxExtractor.h"
#include "DWTC.h"
#include "BeatExtractor.h"
#include "SvFeatExtractor.h"
#include "MultiPitchExtractor.h"
#include "GenreExtractor.h"
#include "MultiPitchFeature.h"
#include "Parallel.h"
#include "Series.h"
#include "ValleyPick.h"
#include "AMDF.h"
#include "knnTree.h"
#include "FFT.h"
#include "AutoCorrelation.h"
#include "Pitch.h"
#include "SpectralPeak.h"
#include "Messager.h"
#include "SKINI.h"
#include "SKINI.msg"
#include <map>
#include <vector>

#include "RtDuplex.h"
#include "RtWvIn.h"
#include "RtWvOut.h"
#include "WvOut.h"
#include "WvIn.h"
#include "waveio.h"



AuFileIO *input;
AuFileIO *fileoutput;
int cursample;
RtWvOut *output;
RtDuplex *inout;
fvec s;
struct soundhdr hdr;
FILE* outfile;
int totalsamples = 0;
int winSize;


inline float tick(float out)
{
  short data = out * 32000;
  totalsamples++;
	fwrite(&data, 2, 1, outfile);
  if(inout != NULL) return inout->tick(out);
	output->tick(out);
	if(cursample%winSize == 0) input->get(s, cursample/winSize);
	return s((cursample++)%winSize);
}



void sort(float *a, int *b, int l, int r)
{
  static float m;
  static int j;
  int i;

  if(r > l) {
    m = a[b[r]]; i = l-1; j = r;
    for(;;) {
      while(a[b[++i]] < m) if(i >= j) break;
      while(a[b[--j]] > m) if(i >= j) break;
      if(i >= j) break;
      std::swap(b[i], b[j]);
    }
    std::swap(b[i], b[r]);
    sort(a, b, l, i-1);
    sort(a, b, i+1, r);
  }
}


inline int round(float r)
{
  if(r - floor(r) <= 0.5 ) return (int)floor(r);
  return (int)ceil(r);
}

int
main(int argc, char *argv[])
{
  //parse input
	winSize = atoi(argv[1]);
	s.create(winSize);
  unsigned int i, j, k, dim=0, hopSize = winSize;
  string winSizeStr = argv[1], collectName = argv[2], nTreeFile = collectName, filename;
	nTreeFile += winSizeStr;
		
	if(argc == 4) filename = argv[3];
	else filename = "empty";
	
  //extract features for each extractor
  int *startnum = NULL, rows;
  BatchExtract *collectExtract;
  collectExtract = new BatchExtract(collectName, winSizeStr, winSizeStr);
  FeatMatrix *collectRes, *dfccRes;
	collectRes = collectExtract->extract("DFCC", &startnum);
	rows = collectRes->rows();
	dfccRes = new FeatMatrix(rows, 80);
	fvec collectVec(rows);
	for(i = 0; i < 80; i++)
	{
	  collectVec = collectRes->col(i+2);
		dfccRes->col((i%20)*4+(i/20), collectVec);
	}
	
	int labelIndexes[rows];
  for(i = 0; i < rows; i++) labelIndexes[i] = collectRes->labelNum(i, 1);
  dfccRes->setLabelNames(collectRes->labelNames());
	/*
  //build search trees or read in if they already exists
  knnTree *tree;
  string numberstring, treeFile;
  struct stat stbuf;
  
  tree = new knnTree(dfccRes, rows, 80, 7);
  treeFile = MARSYAS_MFDIR;
	treeFile += "/trees/";
	treeFile += nTreeFile + ".mtf";
  if(stat(treeFile.c_str(), &stbuf))
  {
    cout << "Building search tree..." << endl;
    tree->makeTree();
    cout << "Writing search tree..." << endl;
    tree->write(treeFile);
  }
  else
  {
    cout << "Reading search tree..." << endl;
    tree->read(treeFile);
  }
  cout << "Tree creation complete." << endl;
*/

  //sort collection by pitch
	int sorted[rows];
	collectVec = collectRes->col(0);
	float *dat = collectVec.getData();
	for(i = 0; i < rows; i++) sorted[i] = i;
  sort(dat, sorted, 0, rows-1);

  //Open up library audio files
  Util util; 
	vector<string> labels = collectRes->labelNames();
  AuFileIO *sample[labels.size()-1];
	for(i = 0; i < labels.size()-1; i++)
	{ 
	  sample[i] = (AuFileIO*)util.sfopen(labels[i+1], MRS_SF_READ);
    sample[i]->initWindow(winSize, hopSize, 0, 0);
	}
	
	fillheader(&hdr, 44100);
  outfile = opensoundout("crap.snd", &hdr);


  
  //Open realtime duplex device.
  Stk::setSampleRate(44100.0);
  try
  {
	  inout = NULL;
		output = NULL;
	  if(filename == "empty") inout = new RtDuplex(2, Stk::sampleRate(), 0, RT_BUFFER_SIZE, 8);
		else
		{
			input = (AuFileIO*)util.sfopen(filename.c_str(), MRS_SF_READ);
			input->initWindow(winSize, winSize, 0, 0);
		  cursample = 0;
			output = new RtWvOut(2, Stk::sampleRate(), 0, RT_BUFFER_SIZE, 8);
		}
  }
  catch (StkError &) { return 0; }

  int knns = 1000, nns[knns], type, label, cs = 20, best, index;
	fvec inwin(winSize), sendwindow(winSize), outwin(winSize), ovlap(winSize);
  fvec irms(1), orms(1), q(cs*4), feats(cs*4+2), tempwin(winSize);
	unsigned int otSz, ovSz = 0;
  float dist[knns], fctr, ovfSz = 0, temp, *data, otfSz, pitch = 0, diff, factor;

  RMS rms(winSize);
  DFCC *dfcc = new DFCC(winSize, 40, cs);
	MagFFT *fft_ = new MagFFT(winSize);

/*
	  ovlap.setval(0.0);
	fvec p(40);
	AuFileIO *test = (AuFileIO*)util.sfopen(filename.c_str(), MRS_SF_READ);
  test->initWindow(winSize, winSize, 0, 0);
	DFCC d(winSize, 40, 20);
	for(i = 0; i < 100; i++)
{
	test->get(inwin, i);
	d.process(inwin, p);
}
exit(0);
*/
  ofstream pitches("pitches");
	for(i = 0; i < rows; i++) pitches << (*collectRes)(i, 0) << endl;
	cout << "go!" << endl;

  nns[0] = 0;
  for(i = 0; i < winSize; i++) inwin(i) = tick(0);
  float mindif, lastpitch = 0, lastpitch2 = 0;
	int closest;
	Hamming ham(winSize, 0);
  int totalwins = input->iterations(), currentwin = 0;
	
  //runtime loop
  while(currentwin < totalwins)
  {    
	  currentwin++;
		sendwindow = inwin;
		dfcc->process(sendwindow, feats);
   
		pitch = feats(0);
	 
		for(i = 0; i < 80; i++) q((i%20)*4+(i/20)) = feats(i+2);
		int low = 0, size = rows-1, mid = 0;
		while(low<=size && ((*collectRes)(sorted[mid], 0) < pitch*.95 || (*collectRes)(sorted[mid], 0) > pitch*1.05))
		{
			mid=((low+size)/2);
		  if((*collectRes)(sorted[mid], 0) < pitch*.95) low=mid+1;
			else if((*collectRes)(sorted[mid], 0) > pitch*1.05) size=mid-1;
		}

	  float distance, tdistance, diff;
		for(i = 0, distance = 0, nns[0] = sorted[mid]; i < 60; i++)
		{
		  diff = q(i) - (*dfccRes)(sorted[mid], i);
		  distance += diff*diff;
		}
		if(distance == 0) distance = 10000;
		for(j = mid+1; j < rows && (*collectRes)(sorted[j], 0) < pitch*1.05; j++)
			//if((*collectRes)(sorted[j], 1) < .5 && (*collectRes)(sorted[j], 35) > .02 && (*collectRes)(sorted[j], 34) < 400)
		  {
		    for(i = 0, tdistance = 0; i < 60 && tdistance < distance; i++)
		    {
		      diff = q(i) - (*dfccRes)(sorted[j], i);
		      tdistance += diff*diff;
		    }
			  if(tdistance < distance && tdistance != 0)
			  {
			    distance = tdistance;
			  	nns[0] = sorted[j];
			  }
		  }
		//cout << j-mid;
		if(mid > 0) for(j = mid-1; j >= 0 && (*collectRes)(sorted[j], 0) > pitch*.95; j--)
			//if((*collectRes)(sorted[j], 1) < .5 && (*collectRes)(sorted[j], 35) > .02 && (*collectRes)(sorted[j], 34) < 400)
		  {
		    for(i = 0, tdistance = 0; i < 60 && tdistance < distance; i++)
		    {
		      diff = q(i) - (*dfccRes)(sorted[j], i);
		      tdistance += diff*diff;
		    }
			  if(tdistance < distance && tdistance != 0)
			  {
			    distance = tdistance;
				  nns[0] = sorted[j];
			  }
		  }
		//cout << " " << mid - j << " " << sorted[mid] << " " << nns[0] << endl;
	 
		factor = (*collectRes)(nns[0], 0) / pitch;
		label = labelIndexes[nns[0]]; 
    sample[label]->get(tempwin, nns[0]-startnum[label]);
		for(i = 0; i < winSize && ceil(i*factor) < winSize; i++)
		{
			index = floor(i*factor);
			diff = i*factor - index;
			outwin(i) = tempwin(index)*(1.0-diff);
			outwin(i) += tempwin(index+1)*diff;
		}
		if(ceil(i*factor) >= winSize && i < winSize)
		{
			index = floor(i*factor);
			diff = i*factor - index;
			if(index < winSize) outwin(i) = tempwin(index)*(1.0-diff);
			label = labelIndexes[nns[0]+1]; 
			sample[label]->get(tempwin, nns[0]-startnum[label]+1);			
			if(index >= winSize) outwin(i) = tempwin(index-winSize)*(1.0-diff);
			outwin(i) += tempwin(index+1-winSize)*diff;
			i++;
			while(i < winSize)
			{
				index = floor(i*factor);
				diff = i*factor - index;
				outwin(i) = tempwin(index-winSize)*(1.0-diff);
				outwin(i) += tempwin(index+1-winSize)*diff;
				i++;
			}
		}
    data = outwin.getData();
    fft_->rfft(data, winSize/2, FFT_FORWARD);
    for(i = 0; i < winSize/2; i++)
		{
		  outwin(2*i) = sqrt(outwin(2*i)*outwin(2*i)+outwin(2*i+1)*outwin(2*i+1));
			outwin(2*i+1) = 0;
		}
    fft_->rfft(data, winSize/2, FFT_INVERSE);
    
    //prepare and send output, get input
    rms.process(inwin, irms);
    rms.process(outwin, orms);
    fctr = irms(0) / orms(0);
    otfSz = 0;
    if(pitch == 0) pitch = 100;
		else pitch = winSize/pitch;
		if(pitch >= winSize/2) pitch = 100;
		otfSz = 0;
		while(otfSz < winSize/2) otfSz += pitch;
		otSz = round(otfSz);
    for(j = 0; j < winSize - otSz; j++)
		  inwin(j) = inwin(j + otSz);
    for(j = 0; j < ovSz; j++)
      inwin(winSize-otSz+j) = tick(((float)j/ovfSz) * fctr * outwin(j) + ovlap(j) * (ovfSz-j)/ovfSz);
    for(j = ovSz; j < otSz; j++)
		  inwin(winSize-otSz+j) = tick(fctr * outwin(j));
    for(j = 0, ovfSz = pitch*floor(winSize/pitch) - otfSz, ovSz = round(ovfSz); j < ovSz; j++)
		  ovlap(j) = outwin(otSz+j) * fctr; 
  }
	closesoundout(outfile, totalsamples);

  return 0;
}


/*    rms.process(inwin, irms);
    rms.process(outwin, orms);
    fctr = irms(0) / orms(0);
    otfSz = 0;
    if(pitch == 0) pitch = 100;
		else pitch = (float)winSize/pitch;
		if(pitch >= winSize/2) pitch = 100;
		otfSz = 0;
		while(otfSz < winSize/2) otfSz += pitch;
		otSz = round(otfSz);
		
		int m = 0;
		float en, maxen = 0;
		for(i = 0; i < lastpitch; i++)
		{
		  for(j = 0, en = 0; j < ovSz; j++) en += outwin(j)*fctr*ovlap(j+i);
		  if(en >= maxen)
			{
			  maxen = en;
				m = i;
		  }
		}
		cout << m << endl;
		
    for(j = 0; j < winSize - otSz - m; j++) inwin(j) = inwin(j + otSz);
		for(j = 0; j < m; j++) inwin(winSize-otSz-m+j) = tick(ovlap(j));
    for(j = 0; j < ovSz; j++)
      inwin(winSize-otSz+j) = tick(((float)j/ovfSz)*fctr*outwin(j) + ovlap(j+m)*(ovfSz-j)/ovfSz);
    for(j = ovSz; j < otSz; j++) inwin(winSize-otSz+j) = tick(fctr * outwin(j));
    for(j = 0, ovfSz = pitch*floor((winSize)/pitch)-otfSz, ovSz = round(ovfSz-pitch); j < round(ovfSz); j++)
		  ovlap(j) = outwin(otSz+j) * fctr; 
	
		ham.process(outwin, outwin);
		outwin*=fctr;
	  for(i = 0; i < winSize/2; i++) inwin(i) = inwin(i+winSize/2);
		for(i = 0; i < winSize/2; i++) inwin(winSize/2+i) = tick(outwin(i)+ovlap(i));
		for(i = 0; i < winSize/2; i++) ovlap(i) = outwin(winSize/2+i);
	*/



/*  
  ovlap.setval(0.0);
	fvec p(40);
	AuFileIO *test = (AuFileIO*)util.sfopen(filename.c_str(), MRS_SF_READ);
  test->initWindow(winSize, winSize, 0, 0);
	DFCC d(winSize, 40, 30);
	for(i = 0; i < 450; i++)
{
	test->get(inwin, i);
	d.process(inwin, p);
}
exit(0);
}
*/


  //get initial samples
/*
	ofstream pitches("pitches"), partials("partials"), amps("amps"), spectrum("spectrum");
	Pitch *pit, *pit2;
	pit = new Pitch("test", winSize);
	
int n = 0;
while(n < 100)
{
 n++;
  for(i=0; i<winSize; i++) inwin(i) = inout->tick(0);
	pit->process(inwin, p);
	pitches << p(0) << endl;
}


	AuFileIO *test = (AuFileIO*)util.sfopen(collectName, MRS_SF_READ);
  test->initWindow(winSize, winSize/2, 0, 0);
	int n = test->iterations();
	pit = new Pitch(winSize);
	
  for(i = 0; i < n; i++) 
  {
    test->get(inwin, i);
    pit->process(inwin, p);
		pitches << p(0) << endl;
  }
		
  exit(0);

*/




/*	Util util;
	AuFileIO *test = (AuFileIO*)util.sfopen(collectName, MRS_SF_READ);
  test->initWindow(winSize, winSize, 0, 0);

	int num = test->iterations();
	for(i = 0; i < num; i++)
	{
	  test->get(inwin, i);
	  systems[0]->process(inwin, p);
		pitches << p(0) << endl;
	}
	 
		 npartials = model.getPartials(inwin, spectrum, quality, partials);
	  fvec ps(npartials*2), ccs(12);
	  for(i = 0; i < npartials; i++)
	  {
	    ps(i) = 2*PI*partials(i)/winSize;
	    ps(i+npartials) = spectrum(partials(i));
  	}
	  dfcc.set(npartials*2, 12);
	  dfcc.process(ps, ccs);
    
 
	  for(i=0; i<winSize/2; i++)
	  {
		  ofpar << i << " " << spectrum(i) << endl;
	    envelope << i << " " << dfcc.get(ccs, 2*PI*i/winSize) << endl;
		}
		ofpar << endl;
		envelope << endl;
	}

  float *data = in.getData(), estim;
	int j;
	magfft_->rfft(data, inSize_/2, FFT_FORWARD);
	for(i = 0; i < inSize_/2; i++)
	{ 
		data[2*i] = 20*log10(sqrt(data[2*i]*data[2*i]+data[2*i+1]*data[2*i+1]));
		logpar << data[2*i] << endl;
		data[2*i+1] = 0;
	}
	magfft_->rfft(data, inSize_/2, FFT_INVERSE);
	int order = 40;
	for(i=0; i<inSize_/2; i++)
	{
	  cepstrum << data[i] << endl;
	  estim = 0;
	  for(j = 0; j <= order; j++) estim += data[j]*cos(j*i*2*PI/inSize_);
	  envelope << estim/2048 << endl;
	}
*/


  /*for(i = 0; i < test->iterations(); i++) 
  {
    test->get(inwin, i);
    systems[0]->process(inwin, *features[0]);
    cout << (*features[0])(0) << endl;
  }
  exit(0);*/


