/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/


/** 
    \class ivec
    \brief Vector of int values

    Array (vector in the numerical sense) of int values. Basic 
arithmetic operations and statistics are supported. 
*/


	

#if !defined(__ivec_h)
#define __ivec_h

#include <stdio.h>
#include <math.h>
#include <assert.h>

#include <string.h>
#include <iostream.h>
using namespace std;

#include "Communicator.h"

class ivec 
{
private:
  int *data_;
  unsigned int size_;
  
public:
  ivec();
  ~ivec();
  ivec(unsigned int size);
  ivec(const ivec& a);
  ivec& operator=(const ivec& a);
  void create(unsigned long size);		// allocate(size) + fill zeros
  void create(int val, unsigned long size);	// allocate(size) + fill val
  void allocate(unsigned long size);	// just allocate(size) random values 
  void setval(unsigned int start, unsigned int end, int val);
  void setval(int val);			// set all entries to val 
  unsigned int size();
  int *get_data();			// dirty for easy integration 
  ivec& operator/=(int val);
  
  ivec& operator+=(const ivec& vec);
  ivec& operator-=(const ivec& vec);
  ivec& operator*=(const ivec& vec);
  ivec& operator/=(const ivec& vec);
  int& operator()(const long i);
  int operator()(const long i) const;
  void debug_info();
  friend ostream& operator<<(ostream&, const ivec&);
  friend istream& operator>>(istream&, ivec&);
  friend ivec operator+(const ivec& vec1, const ivec& vec2);
  friend ivec operator-(const ivec& vec1, const ivec& vec2);
  float mean();
  float std();
  float var();
  
  void sqr();
  void send(Communicator *com);
};


inline 
ivec&
ivec::operator/=(int val)
{
  for (unsigned int i=0; i<size_; i++)
    data_[i] /= val;
  return *this;
}

  
inline
ivec& 
ivec::operator+=(const ivec& vec)
{
  for (unsigned int i=0; i<size_; i++)
    data_[i] += vec.data_[i];
  return *this;
}

inline
ivec& 
ivec::operator-=(const ivec& vec)
{
  for (unsigned int i=0; i<size_; i++)
    data_[i] -= vec.data_[i];
  return *this;
}


inline
ivec& 
ivec::operator*=(const ivec& vec)
{
  for (unsigned int i=0; i<size_; i++)
    data_[i] *= vec.data_[i];
  return *this;
}

inline
ivec& 
ivec::operator/=(const ivec& vec)
{
  for (unsigned int i=0; i<size_; i++)
    data_[i] /= vec.data_[i];
  return *this;
}
  

inline 
int ivec::operator()(const long i) const
{
  return data_[i];
}


inline 
int& ivec::operator()(const long i)
{
  return data_[i];
}



#endif
