

/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/* mkcollection:
   Create a MARSYAS collection from directories and files 
*/

#if HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>

#ifndef WIN32
#include <dirent.h>
#else 
#inlcude <io.h> 
#endif
#include "FeatMatrix.h"
#include "MSignal.h"
#include "fmatrix.h"
#include "SoundFileIO.h"
#include "defs.h"
#include "Util.h"
#include "Gain.h"
#include "Hamming.h"
#include "FeatExtractor.h"
#include "Series.h"
#include "FileName.h"
#include "Collection.h"

#include <string.h>
using namespace std;


int accept(string str)
{
  FileName fname(str);
  string wav("wav");
  string au("au");
  string mp3("mp3");
  
  if ((fname.ext() == wav)
      ||(fname.ext() == au)
      ||(fname.ext() == mp3))
    return 1;
  else return 0;
  
}


void read(Collection& cl, string dir)
{
  cout << "Adding Contents of Directory = " << dir << "\n to collection" << 
       cl.name() << endl;
  DIR* dirp;
  struct dirent *dp;
  dirp = opendir(dir.c_str());
  if (dirp == NULL)
    cout << "Problem with opening directory " << dir << endl;
  
  while ((dp = readdir(dirp))) {
    if (accept(dp->d_name))
      {
	printf("Adding directory entry %s\n", dp->d_name);
	cl.add(dir + dp->d_name);
      }
  }
}

	
int 
main(int argc, char **argv)
{
  int secs;
  if (argc ==1)
    {
      cerr << "Usage :" << argv[0] << " collectionName directory(ies) \n";
      exit(1);
    }
  string collectionName;
  collectionName = argv[1];
  
  cout << "CollectionName = " << collectionName << endl;
  Collection cl;
  FileName fname(collectionName);
  // remove .mf
  collectionName = collectionName.substr(0, collectionName.size()-3); 

  cl.setName(collectionName);
  
  unsigned int i;
  
  for (i=0; i < argc-2; i++)
    {
      read(cl, argv[i+2]);
    }
  string collectionstr = MARSYAS_MFDIR;
  collectionstr += "/collections/";
  collectionstr += collectionName;
  collectionstr += ".mf";
  cl.write(collectionstr);
  cout << "Wrote collection " << collectionstr << endl;
  return 0;
}

	
