/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/


#if HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include "FeatMatrix.h"
#include "MSignal.h"
#include "fmatrix.h"
#include "SoundFileIO.h"
#include "defs.h"
#include "Util.h"
#include "Gain.h"
#include "Hamming.h"
#include "MagFFT.h"
#include "FeatExtractor.h"
#include "Series.h"
#include "MagDB.h"
#include "FFT.h"

#ifndef WIN32
#include "Audio.h"
#endif 
#include "system/AudioDummy.h"


void 
fold(fvec& win, fvec& awin, fvec& fftwin, int in)
{
  int i;
  int winSize = win.size();
  int fftSize = fftwin.size();
  fftwin.setval(0.0);
  while (in < 0)
    in += fftSize;
  in %= fftSize;
  for (i=0; i < winSize; i++)
    {
      fftwin(in) += win(i) * awin(i);
      if (++in == fftSize)
	in = 0;
    }
}


void 
makeWindows(fvec& awin, fvec& swin, unsigned int winSize, 
	    unsigned int fftSize, unsigned int interSize)
{
  
  unsigned int i;
  for (i=0; i < winSize; i++)
    {
      awin(i) = 0.54 - 0.46 * cos(TWO_PI * i / (winSize-1));
      swin(i) = 0.54 - 0.46 * cos(TWO_PI * i / (winSize-1));
    }
  if (winSize > fftSize) 
    {
      float x;
      x = -((int)winSize -1)/2.0;
      for (i=0; i < winSize; i++, x+= 1.0)
	{
	  if ( x != 0.0) 
	    {
	      awin(i) *= fftSize * sin(PI * x / fftSize) / (PI * x);
	      swin(i) *= interSize * sin(PI * x / interSize) / (PI * x);
	    }
	}
    }
  
  /* Normalize for unity gain */
  float sum = 0.0;
  
  for (i=0; i < winSize; i++)
    {
      sum += awin(i);
    }
  for (i=0; i < winSize; i++)
    {
      float afac = 2.0 /sum;
      float sfac = winSize > fftSize ? 1.0/ afac : afac;
      awin(i) *= afac;
      swin(i) *= sfac;
    }
  
  if (winSize <= fftSize) 
    {
      sum = 0.0;
      for (i = 0; i < winSize; i+= interSize) 
	{
	  sum += swin(i) * swin(i);
	}
      for (i=0; i < winSize; i++)
	swin(i) /= sum;
    }
  
  
  
}

void 
oscbank()
{
}



void
overlapadd(fvec& fftwin, fvec& swin, fvec& out, int on)
{
  int i;
  int fftSize = fftwin.size();
  int winSize = swin.size();
  while (on < 0)
    {
      on += fftSize;
    }
  on  %= fftSize;
  for (i=0; i < winSize; i++)
    {
      out(i) += fftwin(on) * swin(i);
      if (++on == fftSize)
	on = 0;
    }
}


      
void shiftout(fvec& out, unsigned int interSize, int on, Audio& audio)
{
  unsigned int i;
  unsigned int winSize = out.size();
  fvec audioout(interSize);
  if (on >= 0)
    for (i=0; i < interSize; i++)
      {
	audioout(i) = out(i);
      }
  audio.put(audioout, 1);
  
  for (i=0; i < winSize - interSize; i++)
    out(i) = out(i+interSize);
  for (i=winSize - interSize; i < winSize; i++)
    out(i) = 0.0;
}

       

int 
main(int argc, char **argv)
{
  unsigned int winSize;
  unsigned int fftSize;
  unsigned int decimSize;
  unsigned int interSize;
  unsigned int pitchFactor;
  unsigned int threshold;
  
  
  if ((argc != 8))
    {
      cerr << "Usage :" << argv[0] << " winSize fftSize decimSize interSize pitchFactor threshold soundfile\n";
      exit(1);
    }
  winSize = atoi(argv[1]);
  fftSize = atoi(argv[2]);
  decimSize = atoi(argv[3]);
  interSize = atoi(argv[4]);
  pitchFactor = atoi(argv[5]);
  threshold = atoi(argv[6]);
  string soundfile = argv[7];
  
  
  cerr << "Phase Vocoder" << endl;
  cerr << "winSize = " << winSize << endl;
  cerr << "fftSize = " << fftSize << endl;
  cerr << "decimation factor  = " << decimSize << endl;
  cerr << "interpolation factor = " << interSize << endl;
  cerr << "pitch factor = " << pitchFactor << endl;
  cerr << "threshold = " << threshold << endl;
  cerr << "soundfile = " << soundfile << endl;
  
  fvec awin(winSize);
  fvec swin(winSize);
  fvec fftwin(fftSize);
  
  
  makeWindows(awin, swin, winSize, fftSize, interSize);
  MSignal *src;
  Util util;
  //src = util.sfopen("/mnt/dos/sound/bwt/Kastoras/kaval_salad.au", SF_READ);
  //src = util.sfopen("../sound/mohonk/gloria.au" , SF_READ);
  //src = util.sfopen("../sound/MUSIC_SPEECH/speech/greek.au" , SF_READ);
  //src = util.sfopen("../sound/MUSIC_SPEECH/speech/ellhnika.au", SF_READ);
  
  src = util.sfopen(soundfile, MRS_SF_READ);
  src->initWindow(winSize, decimSize, 0,0);
  Audio audio(22050, 1, MRS_AUDIO_PLAY, interSize);
  unsigned int i,j;

  fvec win(src->winSize());

  int in = - winSize;
  int on = (in * interSize) / decimSize;
  FFT fft(fftSize, interSize, decimSize, src->srate());
  fvec channels(fftSize + 2);
  fvec magnitude(fftSize/2 + 1);
  fvec phase(fftSize/2 + 1);
  fvec output(winSize);
  
  for (i=0; i < src->iterations(); i++)
    {
      in += decimSize;
      src->get(win,i);
      fold(win, awin, fftwin, in);
      fft.process(fftwin, channels);
      fft.unprocess(channels, fftwin);
      overlapadd(fftwin, swin, output, on);
      shiftout(output, interSize, on, audio);
    }
  return 0;
}
