


/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/* rtime:
 */


#include <stdio.h>
#include "MSignal.h"
#include "SoundFileIO.h"
#include "defs.h"
#include "Util.h"
#include "Classifier.h" 
#include "Gaussian.h" 
#include "GMM.h" 
#include "Spectral.h"
#include "MemMFCC.h"
#include "Client.h"

#ifndef WIN32
#include "Audio.h"
#else
#include "system/AudioDummy.h"
#endif

#define PORTNUM 50001
#define HOSTNAME "epikouros.cs.princeton.edu"

void 
rtime_play(string filename)
{
  
  Gaussian classifier1;
  string clname1 = MARSYAS_MFDIR;
  clname1 += "/classifiers/msFFT.mcf";
  cout << "Clname = " << clname1 << endl;
  ifstream classifierFrom1(clname1.c_str());
  classifierFrom1 >> classifier1;
  
  Gaussian classifier2;
  string clname2 = MARSYAS_MFDIR;
  clname2 += "/classifiers/mfsMFCC.mcf";
  ifstream classifierFrom2(clname2.c_str());
  classifierFrom2 >> classifier2;
  
  Gaussian classifier3;
  string clname3 = MARSYAS_MFDIR;
  clname3 += "/classifiers/genreFFT.mcf";
  ifstream classifierFrom3(clname3.c_str());
  classifierFrom3 >> classifier3;
  


  char in_message[256];
  Client client;
  client.start(HOSTNAME, PORTNUM);


  fvec confidence1(classifier1.classSize());
  fvec pconfidence1(classifier1.classSize());
  fvec confidence2(classifier2.classSize());
  fvec pconfidence2(classifier2.classSize());
  fvec confidence3(classifier3.classSize());
  fvec pconfidence3(classifier3.classSize());
  
  confidence1.setval(0.0);
  pconfidence1.setval(0.0);
  confidence2.setval(0.0);
  pconfidence2.setval(0.0);
  confidence3.setval(0.0);
  pconfidence3.setval(0.0);
  
  fvec confidence(13);
  confidence.setval(0.0);
  confidence(0) = 0.0;
  confidence(2) = 0.0;
  confidence(7) = 0.0;
  
  char message[256];

  sprintf(message, "%f %f %f %f %f %f %f %f %f %f %f %f %f\n",
	  confidence(0), confidence(1), confidence(2),
	  confidence(3), confidence(4), confidence(5),
	  confidence(6), confidence(7), confidence(8),
	  confidence(9), confidence(10), confidence(11), confidence(12));
  client.send_message(message);  
  client.receive_message(in_message);
  cout << "Press key when ready " << endl;
  
  getchar();
  
  MSignal *src;
  Util util;
  src = util.sfopen(filename, MRS_SF_READ);
  if (src == NULL) 
    exit(1);
  src->initWindow(512, 512, 0, 0);
  fvec window(src->winSize());
  Audio audio(src->srate(),1, MRS_AUDIO_PLAY, src->winSize());
  Spectral spectral(src);
  MemMFCC memMFCC(src);

  Spectral *spec1 = new Spectral(src);
  MemMFCC  *mfcc1 = new MemMFCC(src);
  vector <System *> fArray;
  fArray.push_back(spec1);
  fArray.push_back(mfcc1);
  Parallel* fftmfcc = new Parallel(fArray);  
  
  fvec spectralFeatures(spectral.outSize());
  fvec mfccFeatures(memMFCC.outSize());

  
  float max1 = 0.0;
  unsigned int maxIndex1=0;
  float max2 = 0.0;
  unsigned int maxIndex2=0;
  float max3 = 0.0;
  unsigned int maxIndex3=0;

  
  

  
  unsigned int numWindows = 20;
  float alpha = 0.65;
  
  unsigned int i,j;
  
  
  for (i=0; i< src->iterations(); i++)
    {
      if ((i % numWindows) == 0)
	{
	  max1 = 0.0;
	  max2 = 0.0;
	  max3 = 0.0;
	  for (j=0; j < classifier1.classSize(); j++)
	    {
	      if (confidence1(j) > max1) 
		{
		  max1 = confidence1(j);
		  maxIndex1 = j;
		}
	      confidence1(j) = (1.0- alpha) * confidence1(j) + alpha * pconfidence1(j);
	      pconfidence1(j) = confidence1(j);	      
	    }
	  
	  for (j=0; j < classifier2.classSize(); j++)
	    {
	      if (confidence2(j) > max1) 
		{
		  max2 = confidence2(j);
		  maxIndex2 = j;
		}
	      confidence2(j) = (1.0- alpha) * confidence2(j) + alpha * pconfidence2(j);
	      pconfidence2(j) = confidence2(j);	      
	    }
	  
	  for (j=0; j < classifier3.classSize(); j++)
	    {
	      if (confidence3(j) > max3) 
		{
		  max3 = confidence3(j);
		  maxIndex3 = j;
		}
	      confidence3(j) = (1.0- alpha) * confidence3(j) + alpha * pconfidence3(j);
	      pconfidence3(j) = confidence3(j);	      
	    }

	  confidence1 /= (float)numWindows;
	  confidence2 /= (float)numWindows;
	  confidence3 /= (float)numWindows;
	  
	  
	  cout << classifier1.className(maxIndex1) << " " << confidence1(maxIndex1)  << endl;
	  cout << classifier2.className(maxIndex2) << " " << confidence2(maxIndex2)  << endl;
	  cout << classifier3.className(maxIndex3) << " " << confidence3(maxIndex3) << endl;
	  
	  
	  
	  for (j=0; j < 3; j++) 
	    confidence(j) = sqrt(confidence1(1) * confidence2(j) + 0.01);
	  for (j=3; j < 13; j++)
	    {
	      confidence(j) = 0.1 * confidence1(0) + 0.8 * confidence3(j-3);
	      // confidence(j) = sqrt(confidence(j) * confidence1(0) + 0.01);
	      // cout << "I AM HERE " << endl;
	      //confidence(j) = confidence3(j-3);
	    }
	  
	  
	  
	  sprintf(message, "%f %f %f %f %f %f %f %f %f %f %f %f %f\n",
		  confidence(0), confidence(1), confidence(2),
		  confidence(3), confidence(4), confidence(5),
		  confidence(6), confidence(7), confidence(8),
		  confidence(9), confidence(10), confidence(11), confidence(12));
	  
	  client.send_message(message);
	  client.receive_message(in_message);
	  

	  confidence1.setval(0.0);
	  confidence2.setval(0.0);
	}
      
      src->get(window,i);
      spectral.process(window, spectralFeatures);
      memMFCC.process(window, mfccFeatures);
      
      confidence1(classifier1.classify(spectralFeatures))++;
      confidence2(classifier2.classify(mfccFeatures))++;
      confidence3(classifier3.classify(spectralFeatures))++;
      audio.put(window,i);
      
    }
  delete src;
}
	
int 
main(int argc, char **argv)
{
  if (argc == 1)
    {
      cerr << "Usage :" << argv[0] << " filenames\n";
      exit(1);
    }
  int i;
  
  for (i=1; i < argc; i++)
    {
      cerr << "Playing " << argv[i] << endl;
      rtime_play(argv[i]);
    }
  
}
