#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>
#include "FeatMatrix.h"
#include "BatchExtract.h"
#include "MSignal.h"
#include "fmatrix.h"
#include "SoundFileIO.h"
#include "defs.h"
#include "Util.h"
#include "Gain.h"
#include "Hamming.h"
#include "MagFFT.h"
#include "FeatExtractor.h"
#include "Series.h"
#include "MagDB.h"
#include "Sine.h"
#include "Spectral.h"
#include "SpectralSegm.h"
#include "FeatExtractor.h"
#include "Shuffler.h" 
#include "MFCC.h"
#include "ExtractorDictionary.h"
#include "Classifier.h"
#include "MemMFCC.h"
#include "LPC.h"
#include "SfxExtractor.h"
#include "DWTC.h"
#include "BeatExtractor.h"
#include "SvFeatExtractor.h"
#include "MultiPitchExtractor.h"
#include "GenreExtractor.h"
#include "MultiPitchFeature.h"
#include "Parallel.h"
#include "Series.h"
#include "ValleyPick.h"
#include "AMDF.h"
#include "knnTree.h"
#include "FFT.h"
#include "AutoCorrelation.h"
#include "Pitch.h"
#include "DFCC.h"
#include "SpectralPeak.h"
#include "Messager.h"
#include "SKINI.h"
#include "SKINI.msg"
#include <map>
#include <vector>

#include "RtDuplex.h"
#include "RtWvIn.h"
#include "RtWvOut.h"
#include "WvOut.h"
#include "WvIn.h"
#include "waveio.h"
#include "PitShift.h"

#define NORMAL 0
#define REPLACE 1

#define PRESENT 0
#define PAST 1

#define ERROR			0
#define ENERGY		1
#define CENTROID	2
#define FLUX			3
#define SPEED			4
#define PITCH			5
#define HOP				6
#define MEMORY		7

#define BUFFERS 4
#define WIDTH	16
#define PASS (toggle[0]^((*feats)(location, 0)>control(0))) && (toggle[1]^((*feats)(location, 1)>control(1))) && (toggle[2]^((*feats)(location, 2)>control(2))) && (toggle[3]^((*feats)(location, 3)>control(3)))
#define TFACTOR (control(SPEED)*control(PITCH))

AuFileIO *input;
int totalwins, location, winSize = 4096, hopSize = 2058;
RtAudio *output;
struct soundhdr hdr;
FILE* outfile;
long *labelIndexes, totalsamples, first[128], last[128], length[128];
AuFileIO **sample;
bool toggle[8] = {false};
int curseg, firstseg, totalsegs, totaltemp, state, ind, label, curwin;
int inTime, pitchTime, callTime, hopTime = .08*44100, channel, *startnum = NULL;
float dat, diff, outTime, buf;
FeatMatrix *feats;
fvec control(8), overlap(winSize), inwin(winSize), win(BUFFERS*hopSize);
fvec max(8), min(8), pit(2), pitchwin(BUFFERS*winSize), outwin(BUFFERS*winSize);
		
float window(float frame, int numo)
{
	if(numo < frame) return 0.5*(float)numo/frame;
	if(numo >= frame*2) return 0.5*(float)(3*frame-1-numo)/frame;
	return 0.5;
}

float sinc(float x)
{
	float temp;
	if(x==0.0) return 1.0;
	temp = PI * x;
  return sin(temp) / (temp);
}

int callback(char* buffer, int buffer_size, void *data);

int
main(int argc, char *argv[])
{
  //parse input
  unsigned int i, j, k, flags = 0;
  string winSizeStr = "4096", filename = argv[1], hopSizeStr = "2048";
  MY_FLOAT byte2, byte3;
	
	for(i = 2; i < argc; i++)
		if(!strcmp(argv[i],"-m")) flags |= STK_MIDI;
    else if (!strcmp(argv[i],"-p")) flags |= STK_PIPE;

	fillheader(&hdr, 44100);
  outfile = opensoundout("crap.snd", &hdr);

  BatchExtract collectExtract(filename, winSizeStr, hopSizeStr);
	FeatMatrix *pitch = collectExtract.extract("PITCH", &startnum);
	feats = collectExtract.extract("SFFT", &startnum);
	fvec collectVec(pitch->rows());
	collectVec = pitch->col(1);
	feats->col(0, collectVec);
  totalwins = feats->rows();
	
	int type, channel, stream;
	labelIndexes = new int[totalwins];
  for(i = 0; i < totalwins; i++) labelIndexes[i] = pitch->labelNum(i, 1);
	vector<string> labels = pitch->labelNames();
  sample = new (AuFileIO*)[labels.size()-1];
	Util util;
	for(i = 0; i < labels.size()-1; i++)
	{ 
	  sample[i] = (AuFileIO*)util.sfopen(labels[i+1], MRS_SF_READ);
    sample[i]->initWindow(winSize, hopSize, 0, 0);
	}
 
	for(i = 0; i < 4; i++) min(i) = max(i) = (*feats)(0, i);
  for(i = 0; i < totalwins; i++)
		for(j = 0; j < 4; j++)
		{
			if((*feats)(i, j) > max(j)) max(j) = (*feats)(i, j);
			if((*feats)(i, j) < min(j)) min(j) = (*feats)(i, j);
		}
	max(ERROR) = 10;
	max(SPEED) = max(PITCH) = 2;
	min(SPEED) = min(MEMORY) = 0;
	min(PITCH) = 0.01;
	max(MEMORY) = 128;
	min(HOP) = 40;
	max(HOP) = 1;
	control = min;
	control(PITCH) = control(SPEED) = 1;
	inTime = hopSize;
	state = PRESENT;
	location = totalsegs = curseg = firstseg = totalsamples = 0;

  bool done = false, changed = false;
  Messager *messager;
  try
  {
	  messager = new Messager(flags);
		output = new RtAudio(&stream, 0, 2, 0, 0, RtAudio::RTAUDIO_FLOAT32, 44100, &hopSize, 4);
		output->setStreamCallback(stream, &callback, (void *)toggle);
		output->startStream(stream);
  }
  catch(RtError &error) {
		error.printMessage();
		return 0;
	}
	char charbuf[300];

  //runtime loop
	printf("ERROR  \tENERGY  CENT  \tFLUX  \tSPEED  \tPITCH  \tHOP  \tMEMORY\n");
	while(!done)
  {    
	  //if(location == totalwins) location=0;
	  while((type = messager->nextMessage()) != 0)
	  {
      if(type < 0)
			{
			  done = 1;
				cout << endl;
				break;
			}
			byte2 = messager->getByteTwo();
			byte3 = messager->getByteThree();
			channel = messager->getChannel();
			if(type != __SK_ControlChange_ || byte2!=7) toggle[channel] = !byte2;
			else control(channel) = min(channel) + (max(channel)-min(channel))*byte3/128.0;
			changed = true;
		}
		if(changed)
		{
		  cout << "\r";
		  sprintf(charbuf, "%.2f  \t%.2f  \t%i  \t%i  \t%.2f  \t%.2f  \t%.2f  \t%i  \t", control(0), control(1),
				(int)control(2), (int)control(3), control(4), control(5), control(6), (int)control(7)); 
		  cout << charbuf;
			changed = false;
		}
		usleep(100000);
  }
	closesoundout(outfile, totalsamples);
  return 0;
}



int callback(char* buffer, int buffer_size, void *data)
{
	int i;
	float *out = (float *)(buffer);
	while(callTime < hopSize)
	{
		if(PASS)
		{
			if(state == PAST)
			{
				if(totalsegs > control(MEMORY)) totalsegs = control(MEMORY);
				curseg = (firstseg+totalsegs)%128;
				if(totalsegs < control(MEMORY)) totalsegs++;
				else firstseg = firstseg++;
				firstseg = firstseg%128;
				first[curseg] = location;
			}
			last[curseg] = location;
			state = PRESENT;
			label = 0; 
			//label = labelIndexes[location]; 
			sample[0]->get(inwin, location-startnum[0]);
		}
		else if(control(MEMORY))
		{
			if(state == PRESENT && toggle[MEMORY]) curseg = totalsegs*1000;
			else if(state == PRESENT) curseg = -1;
			if(state == PRESENT || curwin > totaltemp)
			{
				if(toggle[MEMORY]) curseg--;
				else curseg++;
				curwin = 0;
				ind = (firstseg+curseg%totalsegs)%128;
				totaltemp = last[ind] - first[ind];
				if(totaltemp < 0) totaltemp += totalwins;
			}
			state = PAST; 
			sample[0]->get(inwin, (first[ind]+curwin)%totalwins-startnum[0]);
			curwin++;
		}
		if(PASS || control(MEMORY))
		{
			for(i = hopSize; i < hopSize*BUFFERS; i++) win(i-hopSize) = win(i);
			for(i = 0; i < hopSize; i++)
			{
			  win(i+(BUFFERS-1)*hopSize) = inwin(i) * i / hopSize + overlap(i);
				overlap(i) = inwin(i+hopSize) * (hopSize-i)/hopSize;
			}
			for(inTime-=hopSize, pitchTime=WIDTH; inTime<hopSize; inTime+=hopTime/TFACTOR, pitchTime+=hopTime)
				for(i=1; i<hopTime*0.75*3; i++) pitchwin(pitchTime+i) += window(hopTime*0.75,i)*win(inTime+i);
			while(floor(outTime) < pitchTime-WIDTH)
			{
				diff = ceil(outTime)-outTime;
				dat = pitchwin(floor(outTime))*diff+pitchwin(ceil(outTime))*(1-diff);
				//i = floor(outTime+WIDTH);
				//if (control(PITCH) < 1.0)
					//for(dat = 0, j = floor(outTime)+1; j < i; j++)
						//dat += outwin(j) * sinc(outTime-j+WIDTH/2);
				// else for(dat = 0, j = floor(outTime)+1; j < i; j++)
				//	dat += outwin(j) * sinc((outTime-j+WIDTH/2)/control(PITCH)) / control(PITCH);
				short d = dat*32000;
				totalsamples++;
				fwrite(&d, 2, 1, outfile);
				outwin(callTime++) = dat;
				outTime += control(PITCH);
			}
			if(pitchTime > WIDTH)
				for(i = pitchTime-WIDTH; i < winSize*BUFFERS; pitchwin(i++) = 0)
					pitchwin(i-pitchTime+WIDTH) = pitchwin(i);
			outTime -= pitchTime-WIDTH;
			//for(i = 0; i < hopSize; i++) outwin(callTime++) = win(i);
		}
		location++;
		if(location == totalwins) location = 0;
  }
	for(i = 0; i < hopSize; i++) out[i*2] = out[i*2+1] = outwin(i);
	while(i < callTime) outwin(i-hopSize) = outwin(i);
	callTime -= hopSize;
	return 0;
}






