/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/* sfplay:
   Command line utility for playing soundfiles. 
*/

#if HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include "MSignal.h"
#include "SoundFileIO.h"
#include "defs.h"
#include "Util.h"

#ifndef WIN32
#include "Audio.h"
#else
#include "AudioWindows.h"
#endif

void 
play(string filename)
{
  MSignal *src;
  Util util;
  src = util.sfopen(filename, MRS_SF_READ);
  if (src == NULL) 
    exit(1);
  src->initWindow(1024, 1024, 0, 0);
  fvec window(src->winSize());
  Audio audio(src->srate(),1, MRS_AUDIO_PLAY, src->winSize());
  unsigned int iterations;
  
  iterations = src->iterations();
  for (unsigned int i=0; i< iterations; i++)
    {
      src->get(window,i);
      audio.put(window,i);
    }
  delete src;
}
	
int 
main(int argc, char **argv)
{
  if (argc == 1)
    {
      cerr << "Usage :" << argv[0] << " filenames\n";
      exit(1);
    }
  int i;
  
  
  for (i=1; i < argc; i++)
    {
      cerr << "Playing " << argv[i] << endl;
      play(argv[i]);
    }
  return 0;
}
