/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/* Author: George Tzanetakis */

/* test_main:
   main entry point for the tests
*/




#include <stdio.h>
#include "FeatMatrix.h"
#include "MSignal.h"
#include "fmatrix.h"
#include "SoundFileIO.h"
#include "defs.h"
#include "Util.h"
#include "Gain.h"
#include "Hamming.h"
#include "MagFFT.h"
#include "FeatExtractor.h"
#include "Series.h"
#include "MagDB.h"
#include "Sine.h"
#include "Spectral.h"
#include "Filter.h"
#include "WaveletPyramid.h"
#include "WaveletStep.h"
#include "Daub4.h"
#include "LPC.h" 
#include "LPCResyn.h"
#include "AutoCorrelation.h"
#include "Collection.h"
#include "KNN.h"
#include "K_MEANS.h"
#include "HalfWaveRectifier.h"
#include "AMDF.h"
#include "ValleyPick.h"
#include "PitchExtractor.h"
#include "GMM.h"

#ifndef WIN32
#include "Plotter.h"
#include "Audio.h"
#else
#include "system/DummyPlotter.h"
#include "system/DummyWindows.h"
#endif

void 
test_fmatrix()
{
  cerr << "Tests for fmatrix object" << endl;
  fmatrix m5(5,5);
  m5.setval(5.0);
  m5(2,2) = 10.0;
  float val = m5(3,3);
  fmatrix m6;
  cerr << m5 << endl;
  cerr << m6 << endl;
  cerr << "Value at 3,3 = " << val << endl;
  cerr << "Value at 2,2 = " << m5(2,2) << endl;
}

void 
test_lpc()
{
  unsigned int i;  
  MSignal *src;
  Util util;
  unsigned int hopSize;
  hopSize = 128;
  src = util.sfopen("allison.au", MRS_SF_READ);
  src->initWindow(512, hopSize, 0, 0);
  fvec window(src->winSize());
  //Audio audio(22050, 1, AUDIO_PLAY, src->hopSize());
  
  unsigned int order = 20;
  LPC lpc(src, order+1);
  LPCResyn lpcresyn(order+2, src->hopSize(), order);
  
  fvec result(order+2);
  fvec output(src->hopSize());
  Plotter plotter;
  AuFileIO *dest;
  dest = new AuFileIO("result.au", MRS_SF_WRITE);
  dest->initMSignal(src->srate(), 1);
  dest->init();
  dest->initWindow(128, 128, 0, 0);
  cerr << "Number of iterations = " << src->iterations() << endl;
  
  for (i=0; i< src->iterations(); i++)
    {
      src->get(window,i);
      lpc.process(window, result);
      lpcresyn.process(result, output);
      dest->put(output,i);
    }

}


  

void 
test_wavelet()
{
  MSignal *src;
  Util util;
  unsigned int i;
  src = util.sfopen("test.au", MRS_SF_READ);
  src->initWindow(512,512,0,0);
  
  fvec window(src->winSize());
  fvec res(src->winSize());
  Daub4 *daub4 = new Daub4(src->winSize(), WVL_FORWARD);
  Daub4 *idaub4 = new Daub4(src->winSize(), WVL_INVERSE);
  WaveletPyramid pyramid(src->winSize(), daub4, WVL_FORWARD);
  WaveletPyramid ipyramid(src->winSize(), idaub4, WVL_INVERSE);
  Audio audio(22050, 1, MRS_AUDIO_PLAY, src->winSize());
  Plotter plotter;
  AuFileIO *dest;
  dest = new AuFileIO("result.au", MRS_SF_WRITE);
  dest->init();
  
  for (i=0; i< src->iterations(); i++)
    {
      src->get(window , i);
      pyramid.process(window, res);
      res.setval(8,512,0.0);
      ipyramid.process(res, window);
      audio.put(window, i);
    }
  
}
  

void 
test_filter(string filename)
{
  fvec nominator(1);
  fvec denom(2);
  nominator(0) = 1.0;
  denom(0) = 1.0;
  denom(1) = -0.9;
  Filter lowpass(nominator, denom, "lpf");


  fvec impulse,result;
  impulse.create(256);
  impulse(0) = 1.0;
  result.create(256);

  AuFileIO *dest;
  dest = new AuFileIO("result.au", MRS_SF_WRITE);
  dest->init();
  MSignal *src;
  Util util;
  unsigned int i;
  src = util.sfopen(filename.c_str(), MRS_SF_READ);
  src->initWindow(512,512,0,0);
  dest->initWindow(512,512,0,0);

  fvec window(src->winSize());
  fvec res(src->winSize());
  fvec diff(src->winSize());
  //Audio audio(22050, 1, AUDIO_PLAY, src->winSize());
  Plotter plotter;
  for (i=0; i< src->iterations(); i++)
    {
      src->get(window , i);
      cout << window << endl;
      //plotter.plot_wait(window);
      //lowpass.process(window, res);
      //audio.put(res, i);
    }
  
  /*lowpass.process(impulse, result);
  Plotter plotter;
  plotter.plot_wait(result);
  result.write("result.mf");
  */
  
}


void 
test_featSeries()
{
  MSignal *src;
  Util util;
  src = util.sfopen("test.au", MRS_SF_READ);
  src->initWindow(1024, 256, 0, 256);
  //MagnitudeDB *magdb = new MagnitudeDB(src->winSize(), src->zeroSize(), -90);
  
  Spectral *spectral = new Spectral(src);
  FeatExtractor fextractor(src, spectral);
  FeatMatrix res(src->iterations(), spectral->outSize());
  fextractor.extract(0, src->iterations(), res);
  cout << res << endl;
}

Sine *
make_Sine(float freq, float amp)
{
  Sine *sine;
  sine = new Sine(22050);
  sine->initWindow(512, 512, 0,0);
  sine->init(freq, amp, 3.0);
  return sine;
}


void 
test_amdf()
{
  MSignal *src;
  Util util;
  src = util.sfopen("/home/gtzan/data/sound/instr/woodwinds/flute/bassfl.061.au", MRS_SF_READ);
  src->initWindow(4096,4096, 0,0);
  Audio audio(src->srate(), 1, MRS_AUDIO_PLAY, src->winSize());
  fvec win(src->winSize());
  fvec res(src->winSize());
  AMDF amdf(src->winSize());
  ValleyPick vpick(src->winSize());
  Plotter plotter;
  fvec period(1);
  PitchExtractor piter(src);
  FeatMatrix pres(1, piter.outSize());
  piter.extract(0, src->iterations(), pres);
}

  
  


void 
test_HalfWaveRectifier()
{
  unsigned int i;
  MSignal *src;
  Util util;
  src = util.sfopen("test.au", MRS_SF_READ);
  src->initWindow(1024, 1024, 0, 0);
  Audio audio(src->srate(), 1, MRS_AUDIO_PLAY, src->winSize());
  fvec win(src->winSize());
  HalfWaveRectifier rect(src->winSize());
  for (i=0; i < src->iterations(); i++)
    {
      src->get(win, i);
      rect.process(win,win);
      audio.put(win,i);
    }
}


void 
test_K_MEANS()
{
  string collectionstr = MARSYAS_MFDIR;
  collectionstr += "/collections/";
  collectionstr += "sfxf.mf";
  string originalstr = MARSYAS_MFDIR;
  originalstr += "/collections/";
  originalstr += "sfx.mf";
  
  K_MEANS kmeans;
  kmeans.init(8);
  kmeans.clusterCollection(collectionstr, originalstr);
  
  
  return;
  
  unsigned int i;
  ifstream from(collectionstr.c_str());
  Collection l;
  from >> l;

  vector <FeatMatrix *> matrices;
  matrices.reserve(200);
  for (i=0; i < l.size(); i++)
    {
      matrices.push_back(new FeatMatrix());
      matrices[i]->read(l.entry(i));
      cout << "Reading FeatMatrix file :" << l.entry(i) << endl;
    }  
  FeatMatrix trainMatrix(matrices);
  kmeans.cluster(&trainMatrix);
  
  trainMatrix.write("kmeans.mf");
}


  
void 
test_KNN()
{
  unsigned int i;
  string collectionstr = MARSYAS_MFDIR;
  collectionstr += "/collections/";
  collectionstr += "MusicSpeechEval.mf";
  ifstream from(collectionstr.c_str());
  Collection l;
  from >> l;

  vector <FeatMatrix *> matrices;
  matrices.reserve(200);
  for (i=0; i < l.size(); i++)
    {
      matrices.push_back(new FeatMatrix());
      matrices[i]->read(l.entry(i));
      cout << "Reading FeatMatrix file :" << l.entry(i) << endl;
    }
  FeatMatrix trainMatrix(matrices);
  KNN knn(5);
  knn.train(&trainMatrix,0);
  FeatMatrix input;
  input.read("test1.mf");
  string className = knn.estimate(input);
  cout << "Estimated class = " << className << endl;
  
  
}



void
test_Sine()
{
  Sine *src1 = make_Sine(440, 0.5);
  Sine *src2 = make_Sine(3.0 * 440, 0.5 / 2.0);
  Sine *src3 = make_Sine(5.0 * 440, 0.5 / 3.0);
  Sine *src4 = make_Sine(7.0 * 440, 0.5 / 3.0);
  
  
  Audio audio(22050, 1, MRS_AUDIO_PLAY, src1->winSize());
  cerr << "What is going on" << endl;  
  fvec window1(src1->winSize());
  fvec window2(src1->winSize());
  fvec window3(src1->winSize());
  fvec window4(src1->winSize());
  Plotter plotter;
  cerr << "I am here " << endl;
  for (unsigned int i=0; i < src1->iterations(); i++)
    {
      src1->get(window1, i);
      src2->get(window2, i);
      src3->get(window3, i);
      src4->get(window4, i);
      //fvec sum = window1 + window2 + window3 + window4;
      fvec sum1 = window1 + window2;
      //plotter.plot_wait(sum1);
      fvec sum2 = window3 + window4;
      //plotter.plot_wait(sum2);
      fvec sum = sum1 + sum2;
      //plotter.plot_wait(sum);
      audio.put(sum, i);
    }
}



void 
test_Extractor()
{
  MSignal *src;
  Util util;
  src = util.sfopen("test.au", MRS_SF_READ);
  src->initWindow(1024, 1024, 0, 256);
  MagFFT *magfft = new MagFFT(src->winSize());
  FeatExtractor fextractor(src, magfft);
  fextractor.extract(0, 10);
}

void 
test_GMM(string filename)
{
  string collectionstr = MARSYAS_MFDIR;
  collectionstr += "/collections/";
  collectionstr += "MusicSpeechEval.mf";
  Collection collection;
  collection.read(collectionstr);
  
  unsigned int collectionSize = collection.size();
  unsigned int i;
  vector <FeatMatrix *> matrices;  
  FeatMatrix input;
  input.read(filename);
  for (i=0; i < collectionSize; i++)
    {
      matrices.push_back(new FeatMatrix());
      matrices[i]->read(collection.entry(i));
      //cout << "Reading FeatMatrix file :" << collection.entry(i) << endl;
    }
  FeatMatrix trainMatrix(matrices);
  GMM classifier(3);
  //classifier.train(&trainMatrix);
  //classifier.write("MusicSpeechGMM.mf");
  classifier.read("MusicSpeechGMM.mf");
  //classifier.write("Read.mf");
  cout << "Estimated class = " << classifier.estimate(input) << endl;

}



void
test_Sound()
{
  MSignal *src;
  Util util;
  unsigned int i;
  src = util.sfopen("new.au", MRS_SF_READ);
  src->initWindow(512, 512, 0, 0);
  src->debug_info();
  MagFFT magfft(src->winSize());
  
  fvec window(magfft.inSize());
  fvec magSpectrum(magfft.outSize());;
  
  cerr << "Window size = " << window.size() << endl;
  
  Plotter plotter;
  Gain gain(window.size(), 0.7);
  cerr << "ITERATIONS = " << src->iterations() << endl;
  Hamming hamming(src->winSize(), src->zeroSize());
  Audio audio(22050, 1, MRS_AUDIO_PLAY, src->winSize());
  
  for (i=0; i< src->iterations(); i++)
    {
      src->get(window , i);
      audio.put(window, i);
      plotter.plot_wait(window);
      hamming.process(window, window);
      magfft.process(window, magSpectrum);
      //plotter.plot_wait(magSpectrum);
    }
  src->get(window, 0);
  //plotter.plot_wait(window);
}



  

void 
test_au()
{
  // Recording
  AuFileIO *dest;
  dest = new AuFileIO("new.au", MRS_SF_WRITE);
  dest->init();
  fvec window(dest->winSize());
  Audio audio(22050, 1, MRS_AUDIO_REC, dest->winSize());
  for (int i=0; i< 100; i++)
    {
      audio.get(window,i);
      dest->put(window,i);
    }  
}


void 
test_FeatMatrix()
{
  FeatMatrix fmatrix(3,3);
  fmatrix(0,0) = 1.0;
  fmatrix(1,1) = 2.0;
  fmatrix(2,2) = 3.0;
  fmatrix(0,1) = 0.0;
  fmatrix(0,2) = 0.0;
  fmatrix(1,0) = 0.0;
  fmatrix(1,2) = 0.0;
  fmatrix(2,0) = 0.0;
  fmatrix(2,1) = 0.0;
  fvec vec(3);
  vec(0) = 5.0;
  vec(1) = 5.0;
  vec(2) = 5.0;
  fmatrix.row(1, vec);
  fmatrix.col(1, vec);
  cerr << fmatrix << endl;
  fvec row2 = fmatrix.row(1);
  cerr << row2 << endl;
}


int
main(int argc, char **argv)
{
  string filename = argv[1];
  //test_fmatrix();
  //test_Sound();
  //test_Extractor();
  //test_filter(filename);
  //test_lpc();
  //test_KNN();
  //test_K_MEANS();
  //test_amdf();
  //test_GMM(filename);
  //test_HalfWaveRectifier();
  //test_wavelet();
  //test_featSeries();
  //test_au();
  //test_Sine();
  //test_FeatMatrix();
  /* FeatMatrix patata;
  ifstream from("../mf/featurefiles/hiphopsv00013.mfc");
  from >> patata;
  cout << patata;
  */
  return 0;
}

