/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/* wavelet:
   Command line utility for experimenting with wavelets.
*/



#if HAVE_CONFIG_H
#include <config.h> 
#endif 


#include <stdio.h>
#include "FeatMatrix.h"
#include "MSignal.h"
#include "fmatrix.h"
#include "SoundFileIO.h"
#include "defs.h"
#include "Util.h"
#include "Gain.h"
#include "Hamming.h"
#include "MagFFT.h"
#include "FeatExtractor.h"
#include "Series.h"
#include "WaveletPyramid.h"
#include "WaveletStep.h" 
#include "Daub4.h"
#include "HalfWaveRectifier.h"
#include "FullWaveRectifier.h"
#include "PeakPicker.h"
#include "AutoCorrelation.h"
#include "AutoCorrelationFFT.h"
#include "OnePole.h"
#include "DownSampler.h"
#include "Norm.h"

#ifndef WIN32
#include "Audio.h"
#else
#include "AudioWindows.h"
#endif

void 
wavelet(string filename, unsigned int winSize, int level)
{
  MSignal *src;
  Util util;
  unsigned int i;
  src = util.sfopen(filename, MRS_SF_READ);
  src->initWindow(winSize,winSize/2,0,0);
  //src->initWindow(winSize,winSize,0,0);
  

  
  fvec window(src->winSize());
  fvec res(src->winSize());

  fvec lpfed(src->winSize());
  Daub4 *daub4 = new Daub4(src->winSize(), WVL_FORWARD);
  Daub4 *idaub4 = new Daub4(src->winSize(), WVL_INVERSE);
  WaveletPyramid pyramid(src->winSize(), daub4, WVL_FORWARD);
  WaveletPyramid ipyramid(src->winSize(), idaub4, WVL_INVERSE);
  // Audio audio(22050, 1, MRS_AUDIO_PLAY, src->winSize());
  unsigned int base = 4;
  unsigned int hlevel = base << level;
  unsigned int llevel = base << (level-1);
  float levels = (log(winSize) / log(2.0));
  levels -=2;
  cout << "# Window size = " << winSize << endl;
  cout << "# Levels = " << levels << endl;
  cout << "# LLevel = " << llevel << endl;
  cout << "# HLevel = " << hlevel << endl;
  cout << "# LFrequancy = " << 11025.0 / pow(2.0, levels - (level-1)) << endl;
  cout << "# HFrequancy = " << 11025.0 / pow(2.0, levels - level) << endl;
  cout << "# Iterations = " << src->iterations() << endl;
  
  //HalfWaveRectifi/er rect(src->winSize());
  FullWaveRectifier rect(src->winSize());
  OnePole lpf(src->winSize(), 0.99);
  unsigned int factor;
  factor = 16;
  DownSampler ds(src->winSize(), factor);
  fvec dsampled(src->winSize()/factor);  
  fvec autoc(src->winSize()/factor);
  AutoCorrelationFFT autocor2(src->winSize()/factor);
  AutoCorrelation autocor1(src->winSize()/factor);
  PeakPicker picker(src->winSize(),factor, 3, 300, 3200, 100);
  Norm norm(src->winSize());
  fvec peaks(src->winSize()/factor);

  vector<fvec> bands;
  vector<fvec> results;
  vector<fvec> lpfbands;
  vector<fvec> rectbands;
  vector<fvec> normbands;
  fvec sum;
  sum.create(src->winSize());
  
  bands.reserve(20);
  results.reserve(20);
  
  for (i=0; i < 5; i++)
    {
      fvec band;
      fvec res;
      fvec lpfed;
      fvec rect;
      fvec normed;
      
      
      lpfed.create(src->winSize());
      band.create(src->winSize());
      res.create(src->winSize());
      rect.create(src->winSize());
      normed.create(src->winSize());

      bands.push_back(band);
      lpfbands.push_back(lpfed);
      results.push_back(res);
      rectbands.push_back(rect);
      normbands.push_back(normed);
    }
  
  unsigned int j;
  fvec silence;
  silence.create(65536);
  

  for (i=0; i< src->iterations(); i++)
    {
      sum.setval(0.0);
      src->get(window , i);
      pyramid.process(window, res);
      
      for (j=0; j < 5; j++)
	{
	  results[j] = res;
	  level = 7+j;
	  hlevel = base << level;
	  llevel = base << (level-1);
	  results[j].setval(hlevel,src->winSize(),0.0);
	  results[j].setval(0,llevel, 0.0);
	  ipyramid.process(results[j], bands[j]);
	  rect.process(bands[j], rectbands[j]);
	  lpf.process(rectbands[j], lpfbands[j]);	  
	  
	  norm.process(rectbands[j], normbands[j]);
	  sum += normbands[j];
	}
      ds.process(sum,dsampled);
      autocor2.process(dsampled, autoc);
      picker.process(autoc, peaks);
      //plotter.plot_wait(peaks);
      //plotter.plot_wait(sum);
      //audio.put(sum,0);
      //audio.put(silence,0);
      cerr << "Iteration " << i << endl;
    }
  
  
  char buffer[100];
  sprintf(buffer, "%s", "histo");
  string name = buffer;

  // add histogram creation  
  //picker.histo(name);
}



int
main(int argc, char **argv)
{
  if (argc != 2) 
    {
      cerr << "Usage : " << argv[0] << " filename [winSize level]\n";
      exit(1);
    }
  string filename = argv[1];
  /*unsigned int winSize = atoi(argv[2]);
  int level = atoi(argv[3]);
  */
  unsigned int winSize = 65536;
  unsigned int level = 0;
  
  wavelet(filename, winSize, level);
  return 0;
}

