#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>
#include "FeatMatrix.h"
#include "BatchExtract.h"
#include "MSignal.h"
#include "fmatrix.h"
#include "SoundFileIO.h"
#include "defs.h"
#include "Util.h"
#include "Gain.h"
#include "Hamming.h"
#include "MagFFT.h"
#include "FeatExtractor.h"
#include "Series.h"
#include "MagDB.h"
#include "Sine.h"
#include "Spectral.h"
#include "SpectralSegm.h"
#include "FeatExtractor.h"
#include "Shuffler.h" 
#include "MFCC.h"
#include "ExtractorDictionary.h"
#include "Classifier.h"
#include "MemMFCC.h"
#include "LPC.h"
#include "SfxExtractor.h"
#include "DWTC.h"
#include "BeatExtractor.h"
#include "SvFeatExtractor.h"
#include "MultiPitchExtractor.h"
#include "GenreExtractor.h"
#include "MultiPitchFeature.h"
#include "Parallel.h"
#include "Series.h"
#include "ValleyPick.h"
#include "AMDF.h"
#include "knnTree.h"
#include "FFT.h"
#include "AutoCorrelation.h"
#include "Pitch.h"
#include "DFCC.h"
#include "SpectralPeak.h"
#include "Messager.h"
#include "SKINI.h"
#include "SKINI.msg"
#include <map>
#include <vector>

#include "RtDuplex.h"
#include "RtWvIn.h"
#include "RtWvOut.h"
#include "WvOut.h"
#include "WvIn.h"
#include "waveio.h"



AuFileIO *input;
int cursample;
RtWvOut *output;
RtDuplex *inout;
struct soundhdr hdr;
FILE* outfile;
int totalsamples = 0;


int
main(int argc, char *argv[])
{
  //parse input
  unsigned int i, j, k, dim=0;
  unsigned int winSize = 4096, hopSize = winSize/2;
  string winSizeStr = "4096", filename = argv[1], hopSizeStr = "2048";
  MY_FLOAT byte2, byte3;

  Util util; 
	fillheader(&hdr, 44100);
  outfile = opensoundout("crap.snd", &hdr);

  int *startnum = NULL;
  BatchExtract *collectExtract;
  collectExtract = new BatchExtract(filename, winSizeStr, hopSizeStr);
	FeatMatrix *pitch = collectExtract->extract("PITCH", &startnum);
	FeatMatrix *sfft = collectExtract->extract("SFFT", &startnum);

  int labelIndexes[pitch->rows()];
  for(i = 0; i < pitch->rows(); i++) labelIndexes[i] = pitch->labelNum(i, 1);
	vector<string> labels = pitch->labelNames();
  AuFileIO *sample[labels.size()-1];
	for(i = 0; i < labels.size()-1; i++)
	{ 
	  sample[i] = (AuFileIO*)util.sfopen(labels[i+1], MRS_SF_READ);
    sample[i]->initWindow(winSize, hopSize, 0, 0);
	}
	
	fvec inwin(winSize), pit(2), overlap(winSize);
  int totalwins = pitch->rows(), currentwin = 0, label;
 
 
  float minpitch, maxpitch, minrms, maxrms, mincent, maxcent, minflux, maxflux, minroll, maxroll;
  minpitch = maxpitch = (*pitch)(0, 1);
	minrms = maxrms = (*sfft)(0, 3);
	mincent = maxcent = (*sfft)(0, 0);
	minroll = maxroll = (*sfft)(0, 1);
	minflux = maxflux = (*sfft)(0, 2);
  for(i = 0; i < totalwins; i++)
	{
	  if((*pitch)(i, 1) > maxpitch) maxpitch = (*pitch)(i, 1);
	  if((*pitch)(i, 1) < minpitch) minpitch = (*pitch)(i, 1);
	  if((*sfft)(i, 0) > maxcent) maxcent = (*sfft)(i, 0);
	  if((*sfft)(i, 0) < mincent) mincent = (*sfft)(i, 0);
	  if((*sfft)(i, 1) > maxroll) maxroll = (*sfft)(i, 1);
	  if((*sfft)(i, 1) < minroll) minroll = (*sfft)(i, 1);
	  if((*sfft)(i, 2) > maxflux) maxflux = (*sfft)(i, 2);
	  if((*sfft)(i, 2) < minflux) minflux = (*sfft)(i, 2);
	  if((*sfft)(i, 3) > maxrms) maxrms = (*sfft)(i, 3);
	  if((*sfft)(i, 3) < minrms) minrms = (*sfft)(i, 3);
  }
	fvec column(pitch->rows());
	maxpitch = 10;
	column = sfft->col(0);
	maxcent = column.mean();
	column = sfft->col(1);
	maxroll = column.mean();
	
	cout << "Pitch:	" << minpitch << "-" << maxpitch<< endl;
	cout << "RMS:		" << minrms << "-" << maxrms<< endl;
	cout << "Cennt:	" << mincent << "-" << maxcent<< endl;
	cout << "Flux:	" << minflux << "-" << maxflux<< endl;
	cout << "Roll:	" << minroll << "-" << maxroll<< endl;
	
	float pcut, ecut, ccut, fcut, rcut;
	int type;
	bool done = false, pless, eless, cless, fless, rless;
  pless=eless=cless=fless=rless=false;

  Stk::setSampleRate(44100.0);
  Messager *messager;
  try
  {
	  messager = new Messager(STK_PIPE);
		cursample = 0;
		output = new RtWvOut(2, Stk::sampleRate(), 0, RT_BUFFER_SIZE, 2);
  }
  catch (StkError &)
  {
    return 0;
  }

  //runtime loop
  while(!done)
  {    
	  if(currentwin == totalwins) currentwin=0;
	  while((type = messager->nextMessage()) != 0)
	  {
      if (type < 0) done = 1;
  		if (type == __SK_ControlChange_)
	  	{
		  	byte2 = messager->getByteTwo();
			  byte3 = messager->getByteThree();
			  switch(messager->getChannel())
			  {
			  	case 0:
					  if(byte2 != 7) pless = !pless;
			  		else pcut = minpitch + (maxpitch-minpitch)*byte3/128.0;
						if(pless) cout << "Pitch < " << pcut << endl;
						else cout << "Pitch > " << pcut << endl;
			  		break;
			  	case 1:
					  if(byte2 != 7) eless = !eless;
			  		else ecut = minrms + (maxrms-minrms)*byte3/128.0;
						if(eless) cout << "RMS < " << ecut << endl;
						else cout << "RMS > " << ecut << endl;
			  		break;			 
					case 2:
					  if(byte2 != 7) cless = !cless;
			  		else ccut = mincent + (maxcent-mincent)*byte3/128.0;
						if(cless) cout << "Cent < " << ccut << endl;
						else cout << "Cent > " << ccut << endl;
			  		break;
					case 3:
					  if(byte2 != 7) fless = !fless;
			  		else fcut = minflux + (maxflux-minflux)*byte3/128.0;
						if(fless) cout << "Flux < " <<fcut << endl;
						else cout << "Flux > " << fcut << endl;
			  		break;
					case 4:
					  if(byte2 != 7) rless = !rless;
			  		else rcut = minroll + (maxroll-minroll)*byte3/128.0;
						if(rless) cout << "Roll < " << rcut << endl;
						else cout << "Roll > " << rcut << endl;
			  		break;
		  		default:
		  		  break;
			  }
			}
		}
    
		if(((pless && (*pitch)(currentwin, 1) < pcut) || (!pless && (*pitch)(currentwin, 1) > pcut))
			&& ((eless && (*sfft)(currentwin, 3) < ecut) || (!eless && (*sfft)(currentwin, 3) > ecut))
			&& ((cless && (*sfft)(currentwin, 0) < ccut) || (!cless && (*sfft)(currentwin, 0) > ccut))
			&& ((fless && (*sfft)(currentwin, 2) < fcut) || (!fless && (*sfft)(currentwin, 2) > fcut))
			&& ((rless && (*sfft)(currentwin, 1) < rcut) || (!rless && (*sfft)(currentwin, 1) > rcut)))
		{
			label = labelIndexes[currentwin]; 
			sample[label]->get(inwin, currentwin-startnum[label]);
		  for(i = 0; i < winSize/2; i++)
		  {
				short data = (inwin(i)*(i*2.0/winSize) + overlap(i)*2*(winSize/2-i)/(winSize))*32000;
				totalsamples++;
				fwrite(&data, 2, 1, outfile);
		    output->tick(inwin(i)*(i*2.0/winSize) + overlap(i)*2*(winSize/2-i)/(winSize));
			  overlap(i) = inwin(i+winSize/2);
		  }
		}
		
	currentwin++;
	
  }
	closesoundout(outfile, totalsamples);

  return 0;
}



